# -*- coding: utf-8 -*-
from abjad.tools import scoretools


def parse_rtm_syntax(rtm):
    r'''Parses RTM syntax.

    ..  container:: example

        **Example 1.** Parses tuplet:

        ::

            >>> rtm = '(1 (1 (1 (1 1)) 1))'
            >>> tuplet = rhythmtreetools.parse_rtm_syntax(rtm)
            >>> tuplet
            FixedDurationTuplet(Duration(1, 4), "c'8 c'16 c'16 c'8")

        ::

            >>> show(tuplet) # doctest: +SKIP

        ..  doctest::

            >>> print(format(tuplet))
            \times 2/3 {
                c'8
                c'16
                c'16
                c'8
            }

    ..  container:: example

        **Example 2.** Also supports fractional durations:

        ::

            >>> rtm = '(3/4 (1 1/2 (4/3 (1 -1/2 1))))'
            >>> tuplet = rhythmtreetools.parse_rtm_syntax(rtm)
            >>> tuplet
            FixedDurationTuplet(Duration(3, 16), "c'8 c'16 { 8/15 c'8 r16 c'8 }")

        ::

            >>> show(tuplet) # doctest: +SKIP

        ..  doctest::

            >>> print(format(_))
            \tweak text #tuplet-number::calc-fraction-text
            \times 9/17 {
                c'8
                c'16
                \tweak edge-height #'(0.7 . 0)
                \times 8/15 {
                    c'8
                    r16
                    c'8
                }
            }

    Returns fixed-duration tuplet or container.
    '''
    from abjad.tools import rhythmtreetools

    result = rhythmtreetools.RhythmTreeParser()(rtm)

    con = scoretools.Container()

    for node in result:
        tuplet = node((1, 4))
        # following line added 2012-08-01. tb.
        tuplet = tuplet[0]
        if tuplet.is_trivial:
            con.extend(tuplet[:])
        else:
            con.append(tuplet)

    if len(con) == 1:
        return con[0]
    return con



# -*- coding: utf-8 -*-
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject


class BowMotionTechnique(AbjadValueObject):
    r'''Bow motion technique.

    ..  container:: example

        **Example 1.** Jété:

        ::

            >>> bow_motion_technique = indicatortools.BowMotionTechnique('jete')
            >>> print(format(bow_motion_technique))
            indicatortools.BowMotionTechnique(
                technique_name='jete',
                )

    ..  container:: example

        **Example 2.** Ordinario:

        ::

            >>> bow_motion_technique = indicatortools.BowMotionTechnique('ordinario')
            >>> print(format(bow_motion_technique))
            indicatortools.BowMotionTechnique(
                technique_name='ordinario',
                )

    Valid technique names include 'ordinario', 'jeté' and 'circular'.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_default_scope',
        '_technique_name',
        )

    _valid_technique_names = (
        'circular',
        'jete',
        'ordinario',
        None,
        )

    ### INITIALIZER ###

    def __init__(
        self,
        technique_name=None,
        ):
        self._default_scope = None
        assert technique_name in self._valid_technique_names
        self._technique_name = technique_name

    ### PUBLIC PROPERTIES ###

    @property
    def default_scope(self):
        r'''Gets default scope of bow motion technique.

        ..  container:: example

            ::

                >>> technique = indicatortools.BowMotionTechnique('jete')
                >>> technique.default_scope is None
                True

        Returns none.
        '''
        return self._default_scope

    @property
    def glissando_style(self):
        r'''Gets glissando style of bow motion technique.

        Returns string.
        '''
        if self.technique_name == 'circular':
            return 'zigzag'
        elif self.technique_name == 'jete':
            return 'dotted-line'
        return 'line'

    @property
    def technique_name(self):
        r'''Gets technique name of bow motion technique.

        Returns string.
        '''
        return self._technique_name



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_make_colored_text_spanner_with_nibs_01():
    r'''Bracket defaults to solid red line with left and right nibs
    and with no nibs at left and right broken edges.
    '''

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")
    leaves = select(staff).by_leaf()
    spanner = spannertools.make_colored_text_spanner_with_nibs()
    attach(spanner, leaves[2:4])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                \override TextSpanner.bound-details.left-broken.text = ##f
                \override TextSpanner.bound-details.left.text = \markup {
                    \draw-line
                        #'(0 . -1)
                    }
                \override TextSpanner.bound-details.right-broken.text = ##f
                \override TextSpanner.bound-details.right.text = \markup {
                    \draw-line
                        #'(0 . -1)
                    }
                \override TextSpanner.color = #red
                \override TextSpanner.dash-fraction = #1
                \override TextSpanner.staff-padding = #2
                \override TextSpanner.thickness = #1.5
                e'8 \startTextSpan
                f'8 \stopTextSpan
                \revert TextSpanner.bound-details
                \revert TextSpanner.color
                \revert TextSpanner.dash-fraction
                \revert TextSpanner.staff-padding
                \revert TextSpanner.thickness
            }
            {
                g'8
                a'8
            }
        }
        '''
        )



# -*- coding: utf-8 -*-
from abjad import *


def test_quantizationtools_QEventSequence_from_millisecond_durations_01():
    r'''Test basic functionality.
    '''

    durations = mathtools.difference_series(
        [x[0] for x in quantizationtools.make_test_time_segments()])
    q_events = quantizationtools.QEventSequence.from_millisecond_durations(
        durations)

    assert q_events == quantizationtools.QEventSequence((
        quantizationtools.PitchedQEvent(
            durationtools.Offset(0, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(163, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(511, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(627, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(1208, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(1417, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(1997, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(3204, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(3297, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(4087, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(4296, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(4830, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(6362, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(6595, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(6687, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(7013, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(7245, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(7872, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(8197, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(8359, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(8638, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(8731, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(8917, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(9288, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(9404, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(10240, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(10356, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(10496, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(10890, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(11169, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(11285, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(11424, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(12307, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(12957, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(13073, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(13653, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(13979, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(14234, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(15256, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(15883, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(16022, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(16649, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(16927, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(17044, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(17276, 1),
            (NamedPitch("c'"),)
            ),
        quantizationtools.TerminalQEvent(
            durationtools.Offset(18483, 1)
            )
    ))


def test_quantizationtools_QEventSequence_from_millisecond_durations_02():
    r'''Silences are not fused.
    '''

    durations = [100, -100, 100, -100, -100, 100]
    q_events = quantizationtools.QEventSequence.from_millisecond_durations(
        durations, fuse_silences=False)

    assert q_events == quantizationtools.QEventSequence((
        quantizationtools.PitchedQEvent(
            durationtools.Offset(0),
            (NamedPitch("c'"),)
            ),
        quantizationtools.SilentQEvent(
            durationtools.Offset(100)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(200),
            (NamedPitch("c'"),)
            ),
        quantizationtools.SilentQEvent(
            durationtools.Offset(300)
            ),
        quantizationtools.SilentQEvent(
            durationtools.Offset(400)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(500),
            (NamedPitch("c'"),)
            ),
        quantizationtools.TerminalQEvent(
            durationtools.Offset(600)
            )
    ))


def test_quantizationtools_QEventSequence_from_millisecond_durations_03():
    r'''Silences are fused.
    '''

    durations = [100, -100, 100, -100, -100, 100]
    q_events = quantizationtools.QEventSequence.from_millisecond_durations(
        durations, fuse_silences=True)

    assert q_events == quantizationtools.QEventSequence((
        quantizationtools.PitchedQEvent(
            durationtools.Offset(0),
            (NamedPitch("c'"),)
            ),
        quantizationtools.SilentQEvent(
            durationtools.Offset(100)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(200),
            (NamedPitch("c'"),)
            ),
        quantizationtools.SilentQEvent(
            durationtools.Offset(300)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(500),
            (NamedPitch("c'"),)
            ),
        quantizationtools.TerminalQEvent(
            durationtools.Offset(600)
            )
    ))



# -*- coding: utf-8 -*-
import copy
from abjad.tools.topleveltools import iterate
from abjad.tools.topleveltools import mutate


def apply_full_measure_tuplets_to_contents_of_measures(
    argument, supplement=None):
    r'''Applies full-measure tuplets to contents of measures in `argument`:

    ::

        >>> staff = Staff([
        ...     Measure((2, 8), "c'8 d'8"),
        ...     Measure((3, 8), "e'8 f'8 g'8")])
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> print(format(staff))
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                \time 3/8
                e'8
                f'8
                g'8
            }
        }

    ::

        >>> scoretools.apply_full_measure_tuplets_to_contents_of_measures(staff)
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> print(format(staff))
        \new Staff {
            {
                \time 2/8
                {
                    c'8
                    d'8
                }
            }
            {
                \time 3/8
                {
                    e'8
                    f'8
                    g'8
                }
            }
        }

    Returns none.
    '''
    from abjad.tools import selectiontools
    from abjad.tools import scoretools

    supplement = selectiontools.Selection(supplement)
    assert isinstance(supplement, selectiontools.Selection)

    for measure in iterate(argument).by_class(scoretools.Measure):
        target_duration = measure._preprolated_duration
        tuplet = scoretools.FixedDurationTuplet(target_duration, measure[:])
        if supplement:
            new_supplement = mutate(supplement).copy()
            tuplet.extend(new_supplement)



# -*- coding: utf-8 -*-
from abjad.tools.rhythmtreetools import RhythmTreeContainer


class QGridContainer(RhythmTreeContainer):
    r'''A container in a ``QGrid`` structure:

    ::

        >>> container = quantizationtools.QGridContainer()

    ::

        >>> print(format(container))
        quantizationtools.QGridContainer(
            preprolated_duration=durationtools.Duration(1, 1),
            )

    Used internally by ``QGrid``.

    Return ``QGridContainer`` instance.
    '''

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### PRIVATE PROPERTIES ###

    @property
    def _leaf_class(self):
        from abjad.tools import quantizationtools
        return quantizationtools.QGridLeaf

    @property
    def _node_class(self):
        from abjad.tools import quantizationtools
        return (type(self), quantizationtools.QGridLeaf)



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchRange___ge___01():

    pitch_range = pitchtools.PitchRange.from_pitches(-39, 48)

    assert -99 <= pitch_range
    assert -39 <= pitch_range
    assert not 0 <= pitch_range
    assert not 48 <= pitch_range
    assert not 99 <= pitch_range


def test_pitchtools_PitchRange___ge___02():

    pitch_range = pitchtools.PitchRange.from_pitches(-39, 48)

    assert NamedPitch(-99) <= pitch_range
    assert NamedPitch(-39) <= pitch_range
    assert not NamedPitch(0) <= pitch_range
    assert not NamedPitch(48) <= pitch_range
    assert not NamedPitch(99) <= pitch_range



# -*- coding: utf-8 -*-
from abjad.tools.graphtools.GraphvizGraph import GraphvizGraph


class GraphvizSubgraph(GraphvizGraph):
    r'''A Graphviz cluster subgraph.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Graphviz'

    __slots__ = (
        '_is_cluster',
        '_edges',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        attributes=None,
        children=None,
        edge_attributes=None,
        is_cluster=True,
        name=None,
        node_attributes=None
        ):
        GraphvizGraph.__init__(
            self,
            attributes=attributes,
            children=children,
            edge_attributes=edge_attributes,
            name=name,
            node_attributes=node_attributes
            )
        self.is_cluster = is_cluster
        self._edges = set([])

    ### PRIVATE PROPERTIES ###

    @property
    def _graphviz_format_pieces(self):
        result = []
        return result

    @property
    def _node_class(self):
        from abjad.tools import graphtools
        return (
            graphtools.GraphvizSubgraph,
            graphtools.GraphvizNode,
            )

    ### PUBLIC PROPERTIES ###

    @property
    def canonical_name(self):
        r'''Canonical name of Graphviz subgraph.

        Returns string.
        '''
        if self.name is not None:
            if self.is_cluster:
                return 'cluster_' + self.name
            return self.name
        prefix = 'subgraph_'
        if self.is_cluster:
            prefix = 'cluster_'
        return prefix + '_'.join(str(x) for x in self.graph_order)

    @property
    def edges(self):
        r'''Edges of Graphviz subgraph.

        Returns tuple.
        '''
        return tuple(self._edges)

    @property
    def is_cluster(self):
        r'''Is true when Graphviz subgraph is a cluster. Otherwise false.

        Returns true or false.
        '''
        return self._is_cluster

    @is_cluster.setter
    def is_cluster(self, arg):
        self._is_cluster = bool(arg)



# -*- coding: utf-8 -*-
import pytest
from abjad.tools import stringtools


def test_to_tridirectional_lilypond_symbol_01():

    assert stringtools.to_tridirectional_lilypond_symbol('^') == '^'
    assert stringtools.to_tridirectional_lilypond_symbol('-') == '-'
    assert stringtools.to_tridirectional_lilypond_symbol('_') == '_'
    assert stringtools.to_tridirectional_lilypond_symbol(Up) == '^'
    assert stringtools.to_tridirectional_lilypond_symbol('default') == '-'
    assert stringtools.to_tridirectional_lilypond_symbol('neutral') == '-'
    assert stringtools.to_tridirectional_lilypond_symbol(Down) == '_'
    assert stringtools.to_tridirectional_lilypond_symbol(1) == '^'
    assert stringtools.to_tridirectional_lilypond_symbol(0) == '-'
    assert stringtools.to_tridirectional_lilypond_symbol(-1) == '_'
    assert stringtools.to_tridirectional_lilypond_symbol(None) == None
    assert pytest.raises(ValueError,
        "stringtools.to_tridirectional_lilypond_symbol('foo')")



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_PianoPedalSpanner___init___01():
    r'''Initialize empty piano pedal spanner.
    '''

    pedal = spannertools.PianoPedalSpanner()
    assert isinstance(pedal, spannertools.PianoPedalSpanner)



# -*- coding: utf-8 -*-
from abjad import *


def test_lilypondfiletools_LilyPondFile_new_01():

    score = Score([Staff("c'8 d'8 e'8 f'8")])
    lilypond_file = lilypondfiletools.LilyPondFile.new(
        music=score,
        date_time_token=False,
        lilypond_language_token=False,
        lilypond_version_token=False,
        )
    lilypond_file.header_block.composer = Markup('Josquin')
    lilypond_file.layout_block.indent = 0
    lilypond_file.paper_block.top_margin = 15
    lilypond_file.paper_block.left_margin = 15

    assert lilypond_file[Score] is score

    assert format(lilypond_file) == stringtools.normalize(
        r'''
        \header {
            composer = \markup { Josquin }
        }

        \layout {
            indent = #0
        }

        \paper {
            left-margin = #15
            top-margin = #15
        }

        \score {
            \new Score <<
                \new Staff {
                    c'8
                    d'8
                    e'8
                    f'8
                }
            >>
        }
        '''
        )



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_set_measure_denominator_and_adjust_numerator_01():

    measure = Measure((3, 8), "c'8 d'8 e'8")
    scoretools.set_measure_denominator_and_adjust_numerator(measure, 16)

    r'''
    {
        \time 6/16
        c'8
        d'8
        e'8
    }
    '''

    assert inspect_(measure).is_well_formed()
    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 6/16
            c'8
            d'8
            e'8
        }
        '''
        )

    scoretools.set_measure_denominator_and_adjust_numerator(measure, 8)

    r'''
    {
        \time 3/8
        c'8
        d'8
        e'8
    }
    '''

    assert inspect_(measure).is_well_formed()
    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 3/8
            c'8
            d'8
            e'8
        }
        '''
        )



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Tuplet_get_timespan_01():

    staff = Staff(r"c'4 d'4 \times 2/3 { e'4 f'4 g'4 }")
    leaves = select(staff).by_leaf()
    score = Score([staff])
    tempo = Tempo((1, 4), 60)
    attach(tempo, leaves[0])

    assert format(score) == stringtools.normalize(
        r'''
        \new Score <<
            \new Staff {
                \tempo 4=60
                c'4
                d'4
                \times 2/3 {
                    e'4
                    f'4
                    g'4
                }
            }
        >>
        '''
        )

    assert inspect_(staff).get_timespan(in_seconds=True) == \
        timespantools.Timespan(0, 4)
    assert inspect_(staff[0]).get_timespan(in_seconds=True) == \
        timespantools.Timespan(0, 1)
    assert inspect_(staff[1]).get_timespan(in_seconds=True) == \
        timespantools.Timespan(1, 2)
    assert inspect_(staff[-1]).get_timespan(in_seconds=True) == \
        timespantools.Timespan(2, 4)



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import graphtools
from abjad.tools import scoretools
from abjad.tools.datastructuretools.TreeNode import TreeNode
from abjad.tools.rhythmtreetools.RhythmTreeMixin import RhythmTreeMixin


class RhythmTreeLeaf(RhythmTreeMixin, TreeNode):
    r'''A rhythm-tree leaf.

    ::

        >>> leaf = rhythmtreetools.RhythmTreeLeaf(
        ...     preprolated_duration=5, is_pitched=True)
        >>> leaf
        RhythmTreeLeaf(
            preprolated_duration=Duration(5, 1),
            is_pitched=True
            )

    Call with a pulse preprolated_duration to generate Abjad leaf objects:

    ::

        >>> result = leaf((1, 8))
        >>> result
        Selection([Note("c'2"), Note("c'8")])

    Generates rests when called, if `is_pitched` is False:

    ::

        >>> rhythmtreetools.RhythmTreeLeaf(
        ...     preprolated_duration=7, is_pitched=False)((1, 16))
        Selection([Rest('r4..')])

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_duration',
        '_is_pitched',
        '_offset',
        '_offsets_are_current',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        preprolated_duration=1,
        is_pitched=True,
        name=None,
        ):
        TreeNode.__init__(self, name=name)
        RhythmTreeMixin.__init__(self, preprolated_duration=preprolated_duration)
        self.is_pitched = is_pitched

    ### SPECIAL METHODS ###

    def __call__(self, pulse_duration):
        r'''Generate Abjad score components:

        ::

            >>> leaf = rhythmtreetools.RhythmTreeLeaf(5)
            >>> leaf((1, 4))
            Selection([Note("c'1"), Note("c'4")])

        Returns sequence of components.
        '''
        pulse_duration = durationtools.Duration(pulse_duration)
        total_duration = pulse_duration * self.preprolated_duration
        if self.is_pitched:
            return scoretools.make_notes(0, total_duration)
        return scoretools.make_rests(total_duration)

    def __graph__(self, **keywords):
        r'''Graphviz graph of rhythm tree leaf.
        '''
        graph = graphtools.GraphvizGraph(name='G')
        node = graphtools.GraphvizNode(
            attributes={
                'label': str(self.preprolated_duration),
                'shape': 'box'
                }
            )
        graph.append(node)
        return graph

    ### PRIVATE PROPERTIES ###

    @property
    def _pretty_rtm_format_pieces(self):
        return [str(self.preprolated_duration)]

    ### PUBLIC PROPERTIES ###

    @property
    def rtm_format(self):
        r'''RTM format of rhythm tree leaf.

        ::

            >>> rhythmtreetools.RhythmTreeLeaf(1, is_pitched=True).rtm_format
            '1'
            >>> rhythmtreetools.RhythmTreeLeaf(5, is_pitched=False).rtm_format
            '-5'

        Returns string.
        '''
        if self.is_pitched:
            return '{!s}'.format(self.preprolated_duration)
        return '-{!s}'.format(self.preprolated_duration)

    ### PUBLIC PROPERTIES ###

    @property
    def is_pitched(self):
        r'''Gets and sets boolean equal to  true if leaf is pitched.
        Otherwise false.

        ::

            >>> leaf = rhythmtreetools.RhythmTreeLeaf()
            >>> leaf.is_pitched
            True

        ::

            >>> leaf.is_pitched = False
            >>> leaf.is_pitched
            False

        Returns true or false.
        '''
        return self._is_pitched

    @is_pitched.setter
    def is_pitched(self, arg):
        self._is_pitched = bool(arg)



# -*- coding: utf-8 -*-
import numbers
from abjad.tools.pitchtools.PitchClass import PitchClass


class NamedPitchClass(PitchClass):
    '''Named pitch-class.

    ..  container:: example

        Initializes from pitch-class name:

        ::

            >>> NamedPitchClass('cs')
            NamedPitchClass('cs')

    ..  container:: example

        Initializes from number of semitones:

        ::

            >>> NamedPitchClass(14)
            NamedPitchClass('d')

    ..  container:: example

        Initializes from named pitch:

        ::

            >>> NamedPitchClass(NamedPitch('g,'))
            NamedPitchClass('g')

    ..  container:: example

        Initializes from numbered pitch:

        ::

            >>> NamedPitchClass(NumberedPitch(15))
            NamedPitchClass('ef')

    ..  container:: example

        Initializes from numbered pitch-class:

        ::

            >>> NamedPitchClass(NumberedPitchClass(4))
            NamedPitchClass('e')

    ..  container:: example

        Initializes from named pitch-class:


        ::

            >>> NamedPitchClass('C#5')
            NamedPitchClass('cs')


    ..  container:: example

        Initializes from named pitch-class:

        ::

            >>> NamedPitchClass(Note("a'8."))
            NamedPitchClass('a')

    ..  container:: example

        Initializes from pitch-class name:

            >>> pitch_class = NamedPitchClass('cs')
            >>> pitch_class
            NamedPitchClass('cs')

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_alteration_in_semitones',
        '_diatonic_pitch_class_number',
        )

    ### INITIALIZER ###

    def __init__(self, argument=None):
        from abjad.tools import pitchtools
        if isinstance(argument, type(self)):
            self._initialize_by_named_pitch_class(argument)
        elif isinstance(argument, pitchtools.NamedPitch):
            self._initialize_by_named_pitch(argument)
        elif pitchtools.Pitch.is_pitch_class_octave_number_string(argument):
            self._initialize_by_pitch_class_octave_number_string(argument)
        elif pitchtools.Pitch.is_pitch_name(argument):
            self._initialize_by_pitch_name(argument)
        elif isinstance(argument, (
            numbers.Number,
            pitchtools.NumberedPitch,
            pitchtools.NumberedPitchClass,
            )):
            self._initialize_by_number(float(argument))
        elif pitchtools.Pitch.is_pitch_carrier(argument):
            self._initialize_by_pitch_carrier(argument)
        elif argument is None:
            self._initialize_by_number(0)
        else:
            message = 'can not instantiate {} from {!r}.'
            message = message.format(type(self).__name__, argument)
            raise TypeError(message)

    ### SPECIAL METHODS ###

    def __add__(self, named_interval):
        r'''Adds `named_interval` to named pitch-class.

        ::

            >>> pitch_class + NamedInterval('+M9')
            NamedPitchClass('ds')

        Return new named pitch-class.
        '''
        from abjad.tools import pitchtools
        dummy_pitch = pitchtools.NamedPitch(self.pitch_class_name, 4)
        pitch = named_interval.transpose(dummy_pitch)
        return type(self)(pitch)

    def __copy__(self, *arguments):
        r'''Copies named pitch-class.

        ::

            >>> import copy
            >>> copy.copy(pitch_class)
            NamedPitchClass('cs')

        Returns new named pitch-class.
        '''
        return type(self)(self)

    def __eq__(self, argument):
        r'''Is true when `argument` can be coerced to a named pitch-class with
        pitch-class name equal to that of this named pitch-class.

        ::

            >>> pitch_class == 'cs'
            True

        Otherwise false:

            >>> pitch_class == 'ds'
            False

        Returns true or false.
        '''
        if isinstance(argument, type(self)):
            return self.pitch_class_name == argument.pitch_class_name
        return self.pitch_class_name == argument

    def __float__(self):
        r'''Changes named pitch-class to a float.

        ::

            >>> float(pitch_class)
            1.0

        Returns float.
        '''
        return float(self.numbered_pitch_class)

    def __format__(self, format_specification=''):
        r'''Formats named pitch-class.

        ..  container:: example

            ::

                >>> format(NamedPitchClass('cs'))
                "pitchtools.NamedPitchClass('cs')"

        Set `format_specification` to `''`, `'lilypond'` or `'storage'`.

        Returns string.
        '''
        superclass = super(NamedPitchClass, self)
        return superclass.__format__(format_specification=format_specification)

    def __hash__(self):
        r'''Hashes named pitch-class.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(NamedPitchClass, self).__hash__()

    def __int__(self):
        r'''Changes named pitch-class to an integer.

        ::

            >>> int(pitch_class)
            1

        Returns nonnegative integer.
        '''
        return int(self.numbered_pitch_class)

    def __lt__(self, argument):
        r'''Is true when `argument` is a named pitch-class with a pitch
        number greater than that of this named pitch-class.

        ..  container:: example

            Compares less than:

            ::

                >>> NamedPitchClass('cs') < NamedPitchClass('d')
                True

        ..  container:: example

            Does not compare less than:

            ::

                >>> NamedPitchClass('d') < NamedPitchClass('cs')
                False

        Raises type error when `argument` is not a named pitch-class.
        '''
        if not isinstance(argument, type(self)):
            message = 'can not compare named pitch-class to {!r}.'
            message = message.format(argument)
            raise TypeError(message)
        return self.number < argument.number

    def __str__(self):
        r'''Gets string representation of named pitch-class.

        ..  container:: example

            ::

                >>> str(pitch_class)
                'cs'

        Returns string.
        '''
        return self.pitch_class_name

    def __sub__(self, argument):
        r'''Subtracts `argument` from named pitch-class.

        ..  container:: example

            ::

                >>> pitch_class - NamedPitchClass('g')
                NamedInversionEquivalentIntervalClass('+aug4')

        Returns named inversion-equivalent interval-class.
        '''
        from abjad.tools import pitchtools
        if not isinstance(argument, type(self)):
            message = 'must be named pitch-class: {!r}.'
            message = message.format(argument)
            raise TypeError(message)
        pitch_1 = pitchtools.NamedPitch(self, 4)
        pitch_2 = pitchtools.NamedPitch(argument, 4)
        mdi = pitchtools.NamedInterval.from_pitch_carriers(
            pitch_1, pitch_2)
        dic = pitchtools.NamedInversionEquivalentIntervalClass(
            mdi.quality_string, mdi.number)
        return dic

    ### PRIVATE METHODS ###

    def _initialize_by_named_pitch(self, argument):
        self._alteration_in_semitones = argument.alteration_in_semitones
        self._diatonic_pitch_class_number = argument.diatonic_pitch_class_number

    def _initialize_by_named_pitch_class(self, argument):
        self._alteration_in_semitones = argument.alteration_in_semitones
        self._diatonic_pitch_class_number = argument.diatonic_pitch_class_number

    def _initialize_by_number(self, argument):
        from abjad.tools import pitchtools
        pitch_class_number = float(argument) % 12
        numbered_pitch_class = pitchtools.NumberedPitchClass(
            pitch_class_number)
        pitch_class_name = numbered_pitch_class.pitch_class_name
        self._initialize_by_pitch_name(pitch_class_name)

    def _initialize_by_pitch_carrier(self, argument):
        from abjad.tools import pitchtools
        named_pitch = pitchtools.NamedPitch.from_pitch_carrier(argument)
        self._initialize_by_named_pitch(named_pitch)

    def _initialize_by_pitch_class_octave_number_string(self, argument):
        from abjad.tools import pitchtools
        group_dict = pitchtools.Pitch._pitch_class_octave_number_regex.match(
            argument).groupdict()
        diatonic_pitch_class_name = group_dict['diatonic_pitch_class_name'].lower()
        symbolic_string = group_dict['symbolic_string']
        self._alteration_in_semitones = \
            pitchtools.Accidental._symbolic_string_to_semitones[
                symbolic_string]
        self._diatonic_pitch_class_number = \
            self._diatonic_pitch_class_name_to_diatonic_pitch_class_number[
                diatonic_pitch_class_name]

    def _initialize_by_pitch_name(self, argument):
        from abjad.tools import pitchtools
        match = pitchtools.Pitch._pitch_name_regex.match(argument.lower())
        if match is None:
            raise ValueError
        groups = match.groups()
        diatonic_pitch_class_name = groups[0]
        abbreviation = groups[1]
        accidental = pitchtools.Accidental(abbreviation)
        self._alteration_in_semitones = accidental.semitones
        self._diatonic_pitch_class_number = \
            self._diatonic_pitch_class_name_to_diatonic_pitch_class_number[
                diatonic_pitch_class_name]

    ### PUBLIC PROPERTIES ###

    @property
    def accidental(self):
        r'''Gets accidental.

        ..  container:: example

            ::

                >>> NamedPitchClass('cs').accidental
                Accidental('s')

        Returns accidental.
        '''
        from abjad.tools import pitchtools
        return pitchtools.Accidental(self._alteration_in_semitones)

    @property
    def alteration_in_semitones(self):
        r'''Gets alteration in semitones.

        ..  container:: example

            ::

                >>> NamedPitchClass('cs').alteration_in_semitones
                1

        Returns nonnegative integer or float.
        '''
        return self._alteration_in_semitones

    # TODO: rename as diatonic_name
    @property
    def diatonic_pitch_class_name(self):
        r'''Gets diatonic pitch-class name.

        ..  container:: example

            ::

                >>> NamedPitchClass('cs').diatonic_pitch_class_name
                'c'

        Returns string.
        '''
        return self._diatonic_pitch_class_number_to_diatonic_pitch_class_name[
            self._diatonic_pitch_class_number]

    @property
    def diatonic_pitch_class_number(self):
        r'''Gets diatonic pitch-class number.

        ::

            >>> NamedPitchClass('cs').diatonic_pitch_class_number
            0

        Returns nonnegative integer.
        '''
        return self._diatonic_pitch_class_number

    # TODO: remove
    @property
    def named_pitch_class(self):
        r'''Gets named pitch-class.

        ..  container:: example

            ::

                >>> NamedPitchClass('cs').named_pitch_class
                NamedPitchClass('cs')

        Returns new named pitch-class.
        '''
        return type(self)(self)

    @property
    def number(self):
        r'''Gets number.

        ..  container:: example

            ::

                >>> NamedPitchClass('cs').number
                1

        Returns nonnegative integer or float.
        '''
        from abjad.tools import pitchtools
        return pitchtools.NumberedPitchClass(self).number

    # TODO: remove
    @property
    def numbered_pitch_class(self):
        r'''Gets numbered pitch-class.

        ..  container:: example

            ::

                >>> NamedPitchClass('cs').numbered_pitch_class
                NumberedPitchClass(1)

        Returns numbered pitch-class.
        '''
        from abjad.tools import pitchtools
        return pitchtools.NumberedPitchClass(self)

    @property
    def pitch_class_label(self):
        r'''Gets pitch-class label.

        ..  container:: example

            ::

                >>> NamedPitchClass('cs').pitch_class_label
                'C#'

        Returns string.
        '''
        return '{}{}'.format(
            self.diatonic_pitch_class_name.upper(),
            self.accidental.symbolic_string,
            )

    # TODO: rename as 'name'
    @property
    def pitch_class_name(self):
        r'''Gets pitch-class name.

        ..  container:: example

            ::

                >>> NamedPitchClass('cs').pitch_class_name
                'cs'

        Returns string.
        '''
        from abjad.tools import pitchtools
        return '{}{}'.format(
            self.diatonic_pitch_class_name,
            pitchtools.Accidental._semitones_to_abbreviation[
                self._alteration_in_semitones],
            )

    # TODO: remove
    @property
    def pitch_class_number(self):
        r'''Gets pitch-class number.

        ..  container:: example

            ::

                >>> NamedPitchClass('cs').pitch_class_number
                1

        Returns integer or float.
        '''
        return (
            self._diatonic_pitch_class_number_to_pitch_class_number[
                self._diatonic_pitch_class_number] +
                self._alteration_in_semitones) % 12

    ### PUBLIC METHODS ###

    def apply_accidental(self, accidental):
        r'''Applies `accidental` to named pitch-class.

        ..  container:: example

            ::

                >>> NamedPitchClass('cs').apply_accidental('qs')
                NamedPitchClass('ctqs')

        Returns new named pitch-class.
        '''
        from abjad.tools import pitchtools
        accidental = pitchtools.Accidental(accidental)
        new_accidental = self.accidental + accidental
        new_name = self.diatonic_pitch_class_name + new_accidental.abbreviation
        return type(self)(new_name)

    def invert(self, axis=None):
        r'''Inverts named pitch-class.

        Not yet implemented.
        '''
        from abjad.tools import pitchtools
        axis = axis or pitchtools.NamedPitch('c')
        axis = pitchtools.NamedPitch(axis)
        this = pitchtools.NamedPitch(self)
        interval = this - axis
        result = axis.transpose(interval)
        result = type(self)(result)
        return result

    def multiply(self, n=1):
        r'''Multiplies named pitch-class by `n`.

        ..  container:: example

            ::

                >>> NamedPitchClass('cs').multiply(3)
                NamedPitchClass('ef')

        Returns new named pitch-class.
        '''
        return type(self)(self.pitch_class_number * n)

    def transpose(self, n=0):
        r'''Transposes named pitch-class by index `n`.

        ..  container:: example

            ::

                >>> named_interval = NamedInterval('major', 2)
                >>> NamedPitchClass('cs').transpose(named_interval)
                NamedPitchClass('ds')

        Returns new named pitch-class.
        '''
        from abjad.tools import pitchtools
        interval = pitchtools.NamedInterval(n)
        pitch = pitchtools.NamedPitch(self, 4)
        pitch = interval.transpose(pitch)
        return type(self)(pitch)



# -*- coding: utf-8 -*-
import sys
from abjad import *


def test_stringtools_strip_diacritics_01():

    if sys.version_info[0] == 2:
        binary_string = 'Dvo\xc5\x99\xc3\xa1k'
    else:
        binary_string = 'Dvořák'
    ascii_string = stringtools.strip_diacritics(binary_string)

    assert ascii_string == 'Dvorak'



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import sequencetools
from abjad.tools import selectiontools
from abjad.tools.topleveltools import attach


def make_multiplied_quarter_notes(
    pitches,
    multiplied_durations,
    ):
    r'''Make quarter notes with `pitches` and `multiplied_durations`:

    ::

        >>> arguments = [[0, 2, 4, 5], [(1, 4), (1, 5), (1, 6), (1, 7)]]
        >>> scoretools.make_multiplied_quarter_notes(*arguments)
        Selection([Note("c'4 * 1"), Note("d'4 * 4/5"), Note("e'4 * 2/3"), Note("f'4 * 4/7")])

    Read `pitches` cyclically where the length of `pitches` is
    less than the length of `multiplied_durations`:

    ::

        >>> arguments = [[0], [(1, 4), (1, 5), (1, 6), (1, 7)]]
        >>> scoretools.make_multiplied_quarter_notes(*arguments)
        Selection([Note("c'4 * 1"), Note("c'4 * 4/5"), Note("c'4 * 2/3"), Note("c'4 * 4/7")])

    Read `multiplied_durations` cyclically where the length of
    `multiplied_durations` is less than the length of `pitches`:

    ::

        >>> arguments = [[0, 2, 4, 5], [(1, 5)]]
        >>> scoretools.make_multiplied_quarter_notes(*arguments)
        Selection([Note("c'4 * 4/5"), Note("d'4 * 4/5"), Note("e'4 * 4/5"), Note("f'4 * 4/5")])

    Returns list of zero or more newly constructed notes.
    '''
    from abjad.tools import scoretools

    multiplied_durations = [
        durationtools.Duration(x) for x in multiplied_durations]
    quarter_notes = []

    sequences = sequencetools.Sequence([pitches, multiplied_durations])
    for pitch, duration in sequences.zip(cyclic=True):
        quarter_note = scoretools.Note(pitch, durationtools.Duration(1, 4))
        duration = durationtools.Duration(duration)
        multiplier = durationtools.Multiplier(
            duration / durationtools.Duration(1, 4))
        attach(multiplier, quarter_note)
        quarter_notes.append(quarter_note)

    quarter_notes = selectiontools.Selection(quarter_notes)
    return quarter_notes



# -*- coding: utf-8 -*-
import math


def is_dotted_integer(argument):
    '''Is true when `argument` is equivalent to a positive integer and
    can be written with zero or more dots. Otherwise false.

    ..  container:: example

        ::

            >>> for argument in range(16):
            ...     print('%s\t%s' % (argument, mathtools.is_dotted_integer(argument)))
            ... 
            0         False
            1         False
            2         False
            3         True
            4         False
            5         False
            6         True
            7         True
            8         False
            9         False
            10        False
            11        False
            12        True
            13        False
            14        True
            15        True

    Returns true or false.

    Integer `n` qualifies as dotted when ``abs(n)`` is of the form
    ``2**j * (2**k - 1)`` with integers ``0 <= j``, ``2 < k``.
    '''
    from abjad.tools import mathtools
    if argument == 0:
        return False
    non_two_product = 1
    non_two_factors = [_ for _ in mathtools.factors(argument) if not _ == 2]
    for non_two_factor in non_two_factors:
        non_two_product *= non_two_factor
    x = math.log(abs(non_two_product) + 1, 2)
    result = 1 < abs(non_two_product) and int(x) == x
    return result



# -*- coding: utf-8 -*-
import inspect
import pytest
from abjad.tools import abjadbooktools
from abjad.tools import datastructuretools
from abjad.tools import indicatortools
from abjad.tools import documentationtools
from abjad.tools import markuptools
from abjad.tools import pitchtools
from abjad.tools import schemetools
from abjad.tools import systemtools
from abjad.tools import tonalanalysistools


_allowed_to_be_empty_string = (
    indicatortools.Articulation,
    markuptools.Postscript,
    pitchtools.Accidental,
    pitchtools.CompoundOperator,
    schemetools.SchemeColor,
    tonalanalysistools.ChordSuspension,
    )

ignored_classes = (
    abjadbooktools.AbjadDirective,
    abjadbooktools.CodeBlock,
    abjadbooktools.CodeOutputProxy,
    abjadbooktools.DoctestDirective,
    abjadbooktools.GraphvizOutputProxy,
    abjadbooktools.ImportDirective,
    abjadbooktools.LilyPondOutputProxy,
    abjadbooktools.RevealDirective,
    abjadbooktools.ShellDirective,
    abjadbooktools.ThumbnailDirective,
    datastructuretools.Enumeration,
    systemtools.StorageFormatAgent,
    systemtools.FormatSpecification,
    systemtools.TestCase,
    )

classes = documentationtools.list_all_abjad_classes(
    ignored_classes=ignored_classes,
    )


@pytest.mark.parametrize('class_', classes)
def test_abjad___str___01(class_):
    r'''All concrete classes have a string representation.

    With the exception of the exception classes. And those classes listed
    explicitly here.
    '''
    if inspect.isabstract(class_):
        return
    if issubclass(class_, Exception):
        return
    instance = class_()
    string = str(instance)
    assert string is not None
    if class_ not in _allowed_to_be_empty_string:
        assert string != ''



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NamedInversionEquivalentIntervalClass__init_by_quality_string_and_number_01():
    r'''Initializeunison.
    '''

    assert str(pitchtools.NamedInversionEquivalentIntervalClass('perfect', 1)) == 'P1'
    assert str(pitchtools.NamedInversionEquivalentIntervalClass('perfect', -1)) == 'P1'
    assert str(pitchtools.NamedInversionEquivalentIntervalClass('augmented', 1)) == 'aug1'
    assert str(pitchtools.NamedInversionEquivalentIntervalClass('augmented', -1)) == 'aug1'
    assert str(pitchtools.NamedInversionEquivalentIntervalClass('diminished', 1)) == 'dim1'
    assert str(pitchtools.NamedInversionEquivalentIntervalClass('diminished', -1)) == 'dim1'


def test_pitchtools_NamedInversionEquivalentIntervalClass__init_by_quality_string_and_number_02():
    r'''Initialize usual cases.
    '''

    assert str(pitchtools.NamedInversionEquivalentIntervalClass('minor', 2)) == '+m2'
    assert str(pitchtools.NamedInversionEquivalentIntervalClass('major', 2)) == '+M2'
    assert str(pitchtools.NamedInversionEquivalentIntervalClass('minor', 3)) == '+m3'
    assert str(pitchtools.NamedInversionEquivalentIntervalClass('major', 3)) == '+M3'
    assert str(pitchtools.NamedInversionEquivalentIntervalClass('perfect', 4)) == '+P4'
    assert str(pitchtools.NamedInversionEquivalentIntervalClass('augmented', 4)) == '+aug4'


def test_pitchtools_NamedInversionEquivalentIntervalClass__init_by_quality_string_and_number_03():
    r'''Initialize inverted cases less than one octave.
    '''

    assert str(pitchtools.NamedInversionEquivalentIntervalClass('major', 7)) == '+m2'
    assert str(pitchtools.NamedInversionEquivalentIntervalClass('minor', 7)) == '+M2'
    assert str(pitchtools.NamedInversionEquivalentIntervalClass('major', 6)) == '+m3'
    assert str(pitchtools.NamedInversionEquivalentIntervalClass('minor', 6)) == '+M3'
    assert str(pitchtools.NamedInversionEquivalentIntervalClass('perfect', 5)) == '+P4'
    assert str(pitchtools.NamedInversionEquivalentIntervalClass('diminished', 5)) == '+aug4'


def test_pitchtools_NamedInversionEquivalentIntervalClass__init_by_quality_string_and_number_04():
    r'''Initialize noninverted cases greater than one octave.
    '''

    assert str(pitchtools.NamedInversionEquivalentIntervalClass('minor', 9)) == '+m2'
    assert str(pitchtools.NamedInversionEquivalentIntervalClass('major', 9)) == '+M2'
    assert str(pitchtools.NamedInversionEquivalentIntervalClass('minor', 10)) == '+m3'
    assert str(pitchtools.NamedInversionEquivalentIntervalClass('major', 10)) == '+M3'
    assert str(pitchtools.NamedInversionEquivalentIntervalClass('perfect', 11)) == '+P4'
    assert str(pitchtools.NamedInversionEquivalentIntervalClass('augmented', 11)) == '+aug4'


def test_pitchtools_NamedInversionEquivalentIntervalClass__init_by_quality_string_and_number_05():
    r'''Initialize inverted cases greater than one octave.
    '''

    assert str(pitchtools.NamedInversionEquivalentIntervalClass('major', 14)) == '+m2'
    assert str(pitchtools.NamedInversionEquivalentIntervalClass('minor', 14)) == '+M2'
    assert str(pitchtools.NamedInversionEquivalentIntervalClass('major', 13)) == '+m3'
    assert str(pitchtools.NamedInversionEquivalentIntervalClass('minor', 13)) == '+M3'
    assert str(pitchtools.NamedInversionEquivalentIntervalClass('perfect', 12)) == '+P4'
    assert str(pitchtools.NamedInversionEquivalentIntervalClass('diminished', 12)) == '+aug4'



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchRange___eq___01():

    range_1 = pitchtools.PitchRange.from_pitches(-39, 48)
    range_2 = pitchtools.PitchRange.from_pitches(-39, 48)

    assert range_1 == range_2
    assert not range_1 != range_2


def test_pitchtools_PitchRange___eq___02():

    range_1 = pitchtools.PitchRange.from_pitches(-39, 48)
    range_2 = pitchtools.PitchRange.from_pitches(0, 48)

    assert not range_1 == range_2
    assert range_1 != range_2



# -*- coding: utf-8 -*-
from abjad import *


def test_rhythmtreetools_RhythmTreeContainer___init___01():

    container = rhythmtreetools.RhythmTreeContainer()

    assert container.children == ()
    assert container.preprolated_duration == 1
    assert container.start_offset == 0
    assert container.parent is None


def test_rhythmtreetools_RhythmTreeContainer___init___02():

    container = rhythmtreetools.RhythmTreeContainer(
        preprolated_duration=2, children=[])

    assert container.children == ()
    assert container.preprolated_duration == 2
    assert container.start_offset == 0
    assert container.parent is None


def test_rhythmtreetools_RhythmTreeContainer___init___03():

    leaf_a = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=1)
    leaf_b = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=2)
    leaf_c = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=1)

    assert leaf_a.start_offset == 0
    assert leaf_a.parent is None

    assert leaf_b.start_offset == 0
    assert leaf_b.parent is None

    assert leaf_c.start_offset == 0
    assert leaf_c.parent is None

    container = rhythmtreetools.RhythmTreeContainer(
        preprolated_duration=4, children=[leaf_a, leaf_b, leaf_c])

    assert container.children == (leaf_a, leaf_b, leaf_c)
    assert container.preprolated_duration == 4
    assert container.start_offset == 0
    assert container.parent is None

    assert leaf_a.start_offset == 0
    assert leaf_a.parent is container

    assert leaf_b.start_offset == 1
    assert leaf_b.parent is container

    assert leaf_c.start_offset == 3
    assert leaf_c.parent is container



# -*- coding: utf-8 -*-
from abjad.tools import systemtools
from abjad.tools.datastructuretools.TreeContainer import TreeContainer


class ReSTDirective(TreeContainer):
    r'''A ReST directive.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'reStructuredText'

    ### INITIALIZER ###

    def __init__(
        self,
        argument=None,
        children=None,
        directive=None,
        name=None,
        options=None,
        ):
        TreeContainer.__init__(self, children=children, name=name)
        assert isinstance(options, (dict, type(None)))
        self._argument = argument
        self._options = {}
        if options is not None:
            self._options.update(options)
        self._directive = directive

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        agent = systemtools.StorageFormatAgent(self)
        names = list(agent.signature_keyword_names)
        for name in ('children', 'name', 'options'):
            if not getattr(self, name, None) and name in names:
                names.remove(name)
        return systemtools.FormatSpecification(
            client=self,
            repr_is_indented=True,
            storage_format_kwargs_names=names,
            )

    ### PRIVATE PROPERTIES ###

    @property
    def _children_rest_format_contributions(self):
        result = []
        for child in self.children:
            result.append('')
            contribution = child._rest_format_contributions
            for x in contribution:
                if x:
                    result.append('   ' + x)
                else:
                    result.append(x)
        return result

    @property
    def _rest_format_contributions(self):
        if self.argument:
            result = ['.. {}:: {}'.format(self.directive, self.argument)]
        else:
            result = ['.. {}::'.format(self.directive)]
        for key, value in sorted(self.options.items()):
            option = '   :{}:'.format(key)
            if value is True:
                pass
            elif value is None or value is False:
                continue
            elif isinstance(value, (list, tuple)):
                option += ' ' + ', '.join(str(x) for x in value)
            elif isinstance(value, (int, float, str)):
                option += ' ' + str(value)
            result.append(option)
        result.extend(self._children_rest_format_contributions)
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def argument(self):
        r'''Gets and sets argument of ReST directive.
        '''
        return self._argument

    @argument.setter
    def argument(self, arg):
        assert isinstance(arg, (str, type(None)))
        self._argument = arg

    @property
    def directive(self):
        r'''Gets and sets directive of ReST directive.
        '''
        return self._directive

    @directive.setter
    def directive(self, argument):
        self._directive = str(argument)

    @property
    def node_class(self):
        r'''Node class of ReST directive.
        '''
        from abjad.tools import documentationtools
        return (
            documentationtools.ReSTDirective,
            documentationtools.ReSTHeading,
            documentationtools.ReSTHorizontalRule,
            documentationtools.ReSTParagraph,
            )

    @property
    def options(self):
        r'''Options of ReST directive.
        '''
        return self._options

    @property
    def rest_format(self):
        r'''ReST format of ReST directive.
        '''
        return '\n'.join(self._rest_format_contributions)



# -*- coding: utf-8 -*-
from abjad import *


def test_systemtools_AbjadConfiguration_set_default_accidental_spelling_01():

    staff = Staff([Note(n, (1, 8)) for n in range(12)])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8
            cs'8
            d'8
            ef'8
            e'8
            f'8
            fs'8
            g'8
            af'8
            a'8
            bf'8
            b'8
        }
        '''
        )


def test_systemtools_AbjadConfiguration_set_default_accidental_spelling_02():

    abjad_configuration.set_default_accidental_spelling('sharps')
    staff = Staff([Note(n, (1, 8)) for n in range(12)])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8
            cs'8
            d'8
            ds'8
            e'8
            f'8
            fs'8
            g'8
            gs'8
            a'8
            as'8
            b'8
        }
        '''
        )

    abjad_configuration.set_default_accidental_spelling('mixed')


def test_systemtools_AbjadConfiguration_set_default_accidental_spelling_03():

    abjad_configuration.set_default_accidental_spelling('flats')
    staff = Staff([Note(n, (1, 8)) for n in range(12)])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8
            df'8
            d'8
            ef'8
            e'8
            f'8
            gf'8
            g'8
            af'8
            a'8
            bf'8
            b'8
        }
        '''
        )

    abjad_configuration.set_default_accidental_spelling('mixed')


def test_systemtools_AbjadConfiguration_set_default_accidental_spelling_04():
    r'''Revert back to default mixed spelling.
    '''

    abjad_configuration.set_default_accidental_spelling('mixed')
    staff = Staff([Note(n, (1, 8)) for n in range(12)])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8
            cs'8
            d'8
            ef'8
            e'8
            f'8
            fs'8
            g'8
            af'8
            a'8
            bf'8
            b'8
        }
        '''
        )



# -*- coding: utf-8 -*-
import copy
from abjad import *


def test_datastructuretools_OrdinalConstant___copy___01():

    constant_1 = datastructuretools.OrdinalConstant('x', -1, 'left')
    constant_2 = copy.deepcopy(constant_1)

    assert isinstance(constant_1, datastructuretools.OrdinalConstant)
    assert isinstance(constant_2, datastructuretools.OrdinalConstant)
    assert not constant_1 is constant_2
    assert constant_1 == constant_2



# -*- coding: utf-8 -*-
import copy
from abjad import *


def test_scoretools_Note___copy___01():
    r'''Copy note.
    '''

    note_1 = Note(12, (1, 4))
    note_2 = copy.copy(note_1)

    assert isinstance(note_1, Note)
    assert isinstance(note_2, Note)
    assert format(note_1) == format(note_2)
    assert note_1 is not note_2


def test_scoretools_Note___copy___02():
    r'''Copy note with LilyPond multiplier.
    '''

    note_1 = Note("c''4")
    attach(Multiplier(1, 2), note_1)
    note_2 = copy.copy(note_1)

    assert isinstance(note_1, Note)
    assert isinstance(note_2, Note)
    assert format(note_1) == format(note_2)
    assert note_1 is not note_2


def test_scoretools_Note___copy___03():
    r'''Copy note with LilyPond grob overrides and LilyPond context settings.
    '''

    note_1 = Note(12, (1, 4))
    override(note_1).staff.note_head.color = 'red'
    override(note_1).accidental.color = 'red'
    set_(note_1).tuplet_full_length = True
    note_2 = copy.copy(note_1)

    assert isinstance(note_1, Note)
    assert isinstance(note_2, Note)
    assert format(note_1) == format(note_2)
    assert note_1 is not note_2


def test_scoretools_Note___copy___04():
    r'''Copy note with grace container.
    '''

    note_1 = Note("c'4")
    grace_container_1 = scoretools.GraceContainer([Note("d'32")], kind='after')
    attach(grace_container_1, note_1)

    assert format(note_1) == stringtools.normalize(
        r'''
        \afterGrace
        c'4
        {
            d'32
        }
        '''
        )

    note_2 = copy.copy(note_1)
    grace_container_2 = inspect_(note_2).get_grace_container()

    assert format(note_2) == stringtools.normalize(
        r'''
        \afterGrace
        c'4
        {
            d'32
        }
        '''
        )

    assert note_1 is not note_2
    assert grace_container_1 is not grace_container_2
    assert grace_container_1.kind == grace_container_2.kind == 'after'



def test_scoretools_Note___copy___05():
    r'''Deepcopy orphan note.
    '''

    note = Note("c'4")
    articulation = Articulation('staccato')
    attach(articulation, note)
    grace = scoretools.GraceContainer("d'16")
    attach(grace, note)
    override(note).note_head.color = 'red'

    assert format(note) == stringtools.normalize(
        r'''
        \grace {
            d'16
        }
        \once \override NoteHead.color = #red
        c'4 -\staccato
        '''
        )

    new_note = copy.deepcopy(note)

    assert not new_note is note
    assert format(new_note) == format(note)


def test_scoretools_Note___copy___06():
    r'''Deepcopy note in score.
    '''

    staff = Staff("c'8 [ c'8 e'8 f'8 ]")
    note = staff[0]
    articulation = Articulation('staccato')
    attach(articulation, note)
    grace = scoretools.GraceContainer("d'16")
    attach(grace, note)
    override(note).note_head.color = 'red'

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \grace {
                d'16
            }
            \once \override NoteHead.color = #red
            c'8 -\staccato [
            c'8
            e'8
            f'8 ]
        }
        '''
        )

    new_note = copy.deepcopy(note)

    assert new_note is not note
    assert inspect_(note).get_parentage().parent is staff
    assert inspect_(new_note).get_parentage().parent is not staff
    assert isinstance(inspect_(new_note).get_parentage().parent, Staff)
    assert format(new_note) == format(note)
    assert format(inspect_(note).get_parentage().parent) == \
        format(inspect_(new_note).get_parentage().parent)


def test_scoretools_Note___copy___07():
    r'''Copying note does note copy hairpin.
    '''

    staff = Staff([Note(n, (1, 8)) for n in range(8)])
    crescendo = Crescendo()
    attach(crescendo, staff[:4])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 \<
            cs'8
            d'8
            ef'8 \!
            e'8
            f'8
            fs'8
            g'8
        }
        '''
        )

    new_note = copy.copy(staff[0])
    staff.append(new_note)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 \<
            cs'8
            d'8
            ef'8 \!
            e'8
            f'8
            fs'8
            g'8
            c'8
        }
        '''
        )
    assert inspect_(staff).is_well_formed()



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_Accidental_is_symbolic_accidental_string_01():

    assert pitchtools.Accidental.is_symbolic_string('#')
    assert pitchtools.Accidental.is_symbolic_string('##')
    assert pitchtools.Accidental.is_symbolic_string('b')
    assert pitchtools.Accidental.is_symbolic_string('bb')
    assert pitchtools.Accidental.is_symbolic_string('+')
    assert pitchtools.Accidental.is_symbolic_string('#+')
    assert pitchtools.Accidental.is_symbolic_string('~')
    assert pitchtools.Accidental.is_symbolic_string('b~')
    assert pitchtools.Accidental.is_symbolic_string('')


def test_pitchtools_Accidental_is_symbolic_accidental_string_02():

    assert not pitchtools.Accidental.is_symbolic_string('foo')
    assert not pitchtools.Accidental.is_symbolic_string(7)
    assert not pitchtools.Accidental.is_symbolic_string(True)
    assert not pitchtools.Accidental.is_symbolic_string('++')
    assert not pitchtools.Accidental.is_symbolic_string('~~')



# -*- coding: utf-8 -*-
from abjad.tools.mathtools.NonreducedRatio import NonreducedRatio


class Ratio(NonreducedRatio):
    '''Ratio.

    ..  container:: example

        Ratio of two numbers:

        ::

            >>> mathtools.Ratio((2, 4))
            Ratio((1, 2))

    ..  container:: example

        Ratio of three numbers:

        ::

            >>> mathtools.Ratio((2, 4, 2))
            Ratio((1, 2, 1))

    '''

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(self, numbers=(1, 1)):
        from abjad.tools import mathtools
        if isinstance(numbers, type(self)):
            numbers = numbers.numbers
        numbers = [int(_) for _ in numbers]
        gcd = mathtools.greatest_common_divisor(*numbers)
        numbers = [_ // gcd for _ in numbers]
        self._numbers = tuple(numbers)

    ### SPECIAL METHODS ###

    def __eq__(self, argument):
        r'''Is true when `argument` equals ratio.

        ..  container:: example

            ::

                >>> ratio_1 = mathtools.Ratio((1, 2, 1))
                >>> ratio_2 = mathtools.Ratio((1, 2, 1))
                >>> ratio_3 = mathtools.Ratio((2, 3, 3))

            ::

                >>> ratio_1 == ratio_1
                True

            ::

                >>> ratio_1 == ratio_2
                True

            ::

                >>> ratio_1 == ratio_3
                False

            ::

                >>> ratio_2 == ratio_1
                True

            ::

                >>> ratio_2 == ratio_2
                True

            ::

                >>> ratio_2 == ratio_3
                False

            ::

                >>> ratio_3 == ratio_1
                False

            ::

                >>> ratio_3 == ratio_2
                False

            ::

                >>> ratio_3 == ratio_3
                True
            
        '''
        superclass = super(Ratio, self)
        return superclass.__eq__(argument)

    def __getitem__(self, argument):
        r'''Gets item or slice identified by `argument`.

        ..  container:: example

            ::

                >>> ratio = mathtools.Ratio((2, 4, 2))
                >>> ratio[1]
                2

        Returns integer or tuple.
        '''
        if isinstance(argument, slice):
            return tuple(self._numbers.__getitem__(argument))
        return self._numbers.__getitem__(argument)

    def __hash__(self):
        r'''Hashes ratio.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(Ratio, self).__hash__()

    def __len__(self):
        r'''Gets length of ratio.

        ..  container:: example

            ::

                >>> ratio = mathtools.Ratio((2, 4, 2))
                >>> len(ratio)
                3

        Returns integer.
        '''
        return len(self._numbers)

    def __str__(self):
        r'''Gets string representation of ratio.

        ..  container:: example

            Ratio of two numbers:

            ::

                >>> str(mathtools.Ratio((2, 4)))
                '1:2'

        ..  container:: example

            Ratio of three numbers:

            ::

                >>> str(mathtools.Ratio((2, 4, 2)))
                '1:2:1'

        Returns string.
        '''
        numbers = (str(x) for x in self.numbers)
        return ':'.join(numbers)

    ### PUBLIC PROPERTIES ###

    @property
    def multipliers(self):
        r'''Gets multipliers of ratio.

        ..  container:: example

            Ratio of two numbers:

            ::

                >>> ratio = mathtools.Ratio((2, 4))
                >>> ratio.multipliers
                (Multiplier(1, 3), Multiplier(2, 3))

        ..  container:: example

            Ratio of three numbers:

            ::

                >>> ratio = mathtools.Ratio((2, 4, 2))
                >>> ratio.multipliers
                (Multiplier(1, 4), Multiplier(1, 2), Multiplier(1, 4))

        Returns tuple of multipliers.
        '''
        from abjad.tools import durationtools
        weight = sum(self.numbers)
        multipliers = [
            durationtools.Multiplier((_, weight))
            for _ in self.numbers
            ]
        multipliers = tuple(multipliers)
        return multipliers

    @property
    def numbers(self):
        r'''Gets numbers of ratio.

        ..  container:: example

            Ratio of two numbers:

            ::

                >>> ratio = mathtools.Ratio((2, 4))
                >>> ratio.numbers
                (1, 2)

        ..  container:: example

            Ratio of three numbers:

            ::

                >>> ratio = mathtools.Ratio((2, 4, 2))
                >>> ratio.numbers
                (1, 2, 1)

        Set to tuple of two or more numbers.

        Returns tuple of two or more numbers.
        '''
        return self._numbers



# -*- coding: utf-8 -*-
from abjad import *


def test_selectiontools_Selection__get_parent_and_start_stop_indices_01():

    staff = Staff("c'8 d'8 e'8 f'8")
    parent, start, stop = staff[2:]._get_parent_and_start_stop_indices()
    assert parent is staff
    assert start == 2
    assert stop == 3


def test_selectiontools_Selection__get_parent_and_start_stop_indices_02():

    staff = Staff("c'8 d'8 e'8 f'8")
    parent, start, stop = staff[:2]._get_parent_and_start_stop_indices()
    assert parent is staff
    assert start == 0
    assert stop == 1


def test_selectiontools_Selection__get_parent_and_start_stop_indices_03():

    selection = selectiontools.Selection()
    parent, start, stop = selection._get_parent_and_start_stop_indices()
    assert parent is None
    assert start is None
    assert stop is None



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_Hairpin___init___01():
    r'''Initialize empty hairpin spanner.
    '''

    hairpin = Hairpin()
    assert isinstance(hairpin, Hairpin)



# -*- coding: utf-8 -*-
from abjad import *


def test_datastructuretools_TreeContainer_append_01():

    leaf_a = datastructuretools.TreeNode()
    leaf_b = datastructuretools.TreeNode()
    leaf_c = datastructuretools.TreeNode()
    leaf_d = datastructuretools.TreeNode()
    container = datastructuretools.TreeContainer()

    assert container.children == ()

    container.append(leaf_a)
    assert container.children == (leaf_a,)

    container.append(leaf_b)
    assert container.children == (leaf_a, leaf_b)

    container.append(leaf_c)
    assert container.children == (leaf_a, leaf_b, leaf_c)

    container.append(leaf_d)
    assert container.children == (leaf_a, leaf_b, leaf_c, leaf_d)

    container.append(leaf_a)
    assert container.children == (leaf_b, leaf_c, leaf_d, leaf_a)



# -*- coding: utf-8 -*-
from abjad.tools.documentationtools.ReSTDirective import ReSTDirective


class ReSTInheritanceDiagram(ReSTDirective):
    r'''A ReST inheritance diagram directive.

    ::

        >>> documentationtools.ReSTInheritanceDiagram(
        ...     argument=spannertools.Beam)
        ReSTInheritanceDiagram(
            argument='abjad.tools.spannertools.Beam.Beam',
            options={
                'private-bases': True,
                }
            )

    ::

        >>> print(_.rest_format)
        .. inheritance-diagram:: abjad.tools.spannertools.Beam.Beam
           :private-bases:

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'reStructuredText'

    ### INITIALIZER ###

    def __init__(self, argument=None, children=None, name=None, options=None):
        if isinstance(argument, type):
            argument = argument.__module__ + '.' + argument.__name__
        new_options = {'private-bases': True}
        if options is not None:
            new_options.update(options)
        ReSTDirective.__init__(
            self,
            argument=argument,
            children=children,
            name=name,
            options=new_options,
            )

    ### PUBLIC PROPERTIES ###

    @property
    def directive(self):
        r'''Directive of ReSt inheritance diagram.

        Returns ``'inheritance-diagram'``.
        '''
        return 'inheritance-diagram'



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools import tonalanalysistools


def test_tonalanalysistools_ChordExtent___eq___01():

    chord_extent = tonalanalysistools.ChordExtent(5)
    u = tonalanalysistools.ChordExtent(5)
    voice = tonalanalysistools.ChordExtent(7)

    assert      chord_extent == chord_extent
    assert      chord_extent == u
    assert not chord_extent == voice

    assert      u == chord_extent
    assert      u == u
    assert not u == voice

    assert not voice == chord_extent
    assert not voice == u
    assert      voice == voice



# -*- coding: utf-8 -*-
from abjad import *
import pytest


def test_scoretools_Container_reverse_01():
    r'''Retrograde works on a depth-0 container with no spanners and no parent.
    '''

    staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    reversed_leaves = reversed(staff[:])
    staff.reverse()

    assert list(reversed_leaves) == list(staff[:])
    assert inspect_(staff).is_well_formed()


def test_scoretools_Container_reverse_02():
    r'''Retrograde works on a depth-0 container with one spanner
    attached and no parent.
    '''

    container = Container("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    leaves = select(container).by_leaf()
    beam = Beam()
    attach(beam, leaves)
    reversed_leaves = reversed(leaves)
    container.reverse()

    assert list(reversed_leaves) == list(container[:])
    assert inspect_(container).is_well_formed()


def test_scoretools_Container_reverse_03():
    r'''Retrograde works on a depth-0 container with one spanner attached
    to its leaves and with no parent.
    '''

    staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    beam = Beam()
    attach(beam, staff[:])
    reversed_leaves = reversed(staff[:])
    staff.reverse()

    assert list(reversed_leaves) == list(staff[:])
    assert beam.components == tuple(staff[:])
    assert inspect_(staff).is_well_formed()


def test_scoretools_Container_reverse_04():
    r'''Retrograde works on a depth-0 container with one spanner
    attached to itself and with a parent.
    '''

    staff = Staff([Measure((4, 4), "c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")] + \
        scoretools.make_repeated_notes(2))
    measure = staff[0]
    beam = Beam()
    attach(beam, measure[:])
    reversed_leaves = reversed(measure[:])
    staff[0].reverse()
    assert list(reversed_leaves) == list(measure[:])
    assert inspect_(staff).is_well_formed()


def test_scoretools_Container_reverse_05():
    r'''Retrograde works on a depth-0 container with one spanner
    attached to its leaves and with a parent.
    '''

    staff = Staff([Measure((4, 4), "c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")] +
        scoretools.make_repeated_notes(2))
    measure = staff[0]
    beam = Beam()
    attach(beam, measure[:])
    reversed_leaves = reversed(measure[:])
    staff[0].reverse()
    assert list(reversed_leaves) == list(measure[:])
    assert beam.components == tuple(measure[:])
    assert inspect_(staff).is_well_formed()


def test_scoretools_Container_reverse_06():
    r'''Retrograde works on a depth-0 container with one spanner
    attached to its parent.
    '''

    notes = [Note("c'8"), Note("d'8")]
    container = Container(
        [Measure((4, 4), "c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")] + notes)
    measure = container[0]
    beam = Beam()
    attach(beam, measure[:])
    reversed_leaves = reversed(measure[:])
    container[0].reverse()
    assert list(reversed_leaves) == list(measure[:])
    assert inspect_(container).is_well_formed()


def test_scoretools_Container_reverse_07():
    r'''Retrograde works on a depth-0 container with one spanner
    attached to its parent's contents.
    '''

    notes = [Note("c'8"), Note("d'8")]
    measure = Measure((4, 4), "c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    staff = Staff([measure] + notes)
    leaves = select(staff).by_leaf()
    beam = Beam()
    attach(beam, leaves)
    reversed_leaves = reversed(measure[:])
    staff[0].reverse()
    assert list(reversed_leaves) == list(measure[:])
    assert inspect_(staff).is_well_formed()


def test_scoretools_Container_reverse_08():

    notes = [Note("c'8"), Note("d'8")]
    measure = Measure((4, 4), "c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    staff = Staff([measure] + notes)
    leaves = select(staff).by_leaf()
    beam = Beam()
    attach(beam, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 4/4
                c'8 [
                d'8
                e'8
                f'8
                g'8
                a'8
                b'8
                c''8
            }
            c'8
            d'8 ]
        }
        '''
        )

    staff.reverse()

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            d'8 [
            c'8
            {
                \time 4/4
                c'8
                d'8
                e'8
                f'8
                g'8
                a'8
                b'8
                c''8 ]
            }
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_scoretools_Container_reverse_09():
    r'''Retrograde works on a depth-2 container with no parent
    and with spanners at all levels.
    '''

    measure_1 = Measure((4, 8), "c'8 d'8 e'8 f'8")
    measure_2 = Measure((3, 8), "c'8 d'8 e'8")
    container = Container([measure_1, measure_2])
    leaves = select(container).by_leaf()
    pedal = spannertools.PianoPedalSpanner()
    attach(pedal, leaves)
    trill = spannertools.TrillSpanner()
    attach(trill, leaves)
    beam_1 = Beam()
    attach(beam_1, measure_1[:])
    beam_2 = Beam()
    attach(beam_2, measure_2[:])
    gliss = spannertools.Glissando()
    attach(gliss, leaves)

    assert format(container) == stringtools.normalize(
        r'''
        {
            {
                \time 4/8
                \set Staff.pedalSustainStyle = #'mixed
                c'8 [ \glissando \sustainOn \startTrillSpan
                d'8 \glissando
                e'8 \glissando
                f'8 ] \glissando
            }
            {
                \time 3/8
                c'8 [ \glissando
                d'8 \glissando
                e'8 ] \sustainOff \stopTrillSpan
            }
        }
        '''
        )

    container.reverse()

    assert format(container) == stringtools.normalize(
        r'''
        {
            {
                \time 3/8
                \set Staff.pedalSustainStyle = #'mixed
                c'8 [ \glissando \sustainOn \startTrillSpan
                d'8 \glissando
                e'8 ] \glissando
            }
            {
                \time 4/8
                c'8 [ \glissando
                d'8 \glissando
                e'8 \glissando
                f'8 ] \sustainOff \stopTrillSpan
            }
        }
        '''
        )

    assert container[0] is measure_2
    assert container[1] is measure_1
    assert len(measure_2) == 3
    assert len(measure_1) == 4
    leaves = tuple(iterate(container).by_leaf())
    assert pedal.components == leaves
    assert gliss.components == leaves
    assert inspect_(container).is_well_formed()



# -*- coding: utf-8 -*-
from abjad.tools import abctools
from abjad.tools import durationtools
from abjad.tools.topleveltools.iterate import iterate


class InspectionAgent(abctools.AbjadObject):
    r'''Inspection agent.

    ..  container:: example

        ::

            >>> staff = Staff("c'4 e'4 d'4 f'4")
            >>> show(staff) # doctest: +SKIP

        ::

            >>> inspect_(staff)
            InspectionAgent(client=Staff("c'4 e'4 d'4 f'4"))

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_client',
        )

    ### INITIALIZER ###

    def __init__(self, client=None):
        from abjad.tools import scoretools
        from abjad.tools import spannertools
        prototype = (scoretools.Component, spannertools.Spanner, type(None))
        if not isinstance(client, prototype):
            message = 'must be component, spanner or none: {!r}.'
            message = message.format(client)
            raise TypeError(message)
        self._client = client

    ### PUBLIC METHODS ###

    def get_annotation(self, name, default=None):
        r'''Gets value of annotation with `name` attached to client.

        Returns `default` when no annotation with `name` is attached
        to client.

        Raises exception when more than one annotation with `name`
        is attached to client.
        '''
        from abjad.tools import indicatortools
        annotations = self.get_indicators(indicatortools.Annotation)
        if not annotations:
            return default
        with_correct_name = []
        for annotation in annotations:
            if annotation.name == name:
                with_correct_name.append(annotation)
        if not with_correct_name:
            return default
        if 1 < len(with_correct_name):
            message = 'multiple annotations with name {!r} attached.'
            message = message.format(name)
            raise Exception(message)
        annotation_value = with_correct_name[0].value
        return annotation_value

    def get_badly_formed_components(self):
        r'''Gets badly formed components in client.

        ..  container:: example

            ::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> staff[1].written_duration = Duration(1, 4)
                >>> beam = spannertools.Beam()
                >>> attach(beam, staff[:])

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    c'8 [
                    d'4
                    e'8
                    f'8 ]
                }

            ::

                >>> inspect_(staff).get_badly_formed_components()
                [Note("d'4")]

            (Beamed quarter notes are not well formed.)

        Returns list.
        '''
        from abjad.tools import systemtools
        manager = systemtools.WellformednessManager()
        violators = []
        for current_violators, total, check_name in manager(self._client):
            violators.extend(current_violators)
        return violators

    def get_components(
        self,
        prototype=None,
        include_self=True,
        ):
        r'''Gets all components of `prototype`
        in the descendants of client.

        Returns client selection.
        '''
        return self._client._get_components(
            prototype=prototype,
            include_self=include_self,
            )

    def get_contents(
        self,
        include_self=True,
        ):
        r'''Gets contents of client.

        Returns sequential selection.
        '''
        return self._client._get_contents(
            include_self=include_self,
            )

    def get_descendants(
        self,
        include_self=True,
        ):
        r'''Gets descendants of client.

        Returns descendants.
        '''
        return self._client._get_descendants(
            include_self=include_self,
            )

    def get_duration(
        self,
        in_seconds=False,
        ):
        r'''Gets duration of client.

        Returns duration.
        '''
        return self._client._get_duration(
            in_seconds=in_seconds,
            )

    def get_effective(
        self,
        prototype=None,
        unwrap=True,
        n=0,
        ):
        r'''Gets effective indicator that matches `prototype`
        and governs client.

        ..  container:: example

            **Example.** Gets components' effective clef:

            ::

                >>> staff = Staff("c'4 d' e' f'")
                >>> attach(Clef('alto'), staff)
                >>> grace_container = scoretools.GraceContainer(
                ...    [Note("fs'16")],
                ...     kind='acciaccatura',
                ...     )
                >>> attach(grace_container, staff[-1])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    \clef "alto"
                    c'4
                    d'4
                    e'4
                    \acciaccatura {
                        fs'16
                    }
                    f'4
                }

            ::

                >>> for leaf in iterate(staff).by_class(with_grace_notes=True):
                ...     clef = inspect_(leaf).get_effective(Clef)
                ...     print(leaf, clef)
                Staff("c'4 d'4 e'4 f'4") Clef(name='alto')
                c'4 Clef(name='alto')
                d'4 Clef(name='alto')
                e'4 Clef(name='alto')
                fs'16 Clef(name='alto')
                f'4 Clef(name='alto')

        Returns indicator or none.
        '''
        return self._client._get_effective(
            prototype=prototype,
            unwrap=unwrap,
            n=n,
            )

    def get_effective_staff(self):
        r'''Gets effective staff of client.

        Returns staff or none.
        '''
        return self._client._get_effective_staff()

    def get_grace_container(
        self,
        kind=None,
        ):
        r'''Gets exactly one grace container of `kind` attached to client.

        Raises error when no grace container of `kind` attaches to client.

        Raises error when more than one grace container of `kind` attaches
        to client.

        Returns grace container.
        '''
        grace_containers = self.get_grace_containers(kind=kind)
        if not grace_containers:
            message = 'no grace containers of {!r} attached.'
            message = message.format(kind)
            raise Exception(message)
        if 1 < len(grace_containers):
            message = 'more than one grace container of {!r} attached.'
            message = message.format(kind)
            raise Exception(message)
        grace_container = grace_containers[0]
        return grace_container

    def get_grace_containers(
        self,
        kind=None,
        ):
        r'''Gets grace containers attached to leaf.

        ..  container:: example

            **Example 1.** Get all grace containers attached to note:

            ::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> grace_container = scoretools.GraceContainer(
                ...     [Note("cs'16")],
                ...     kind='grace',
                ...     )
                >>> attach(grace_container, staff[1])
                >>> after_grace = scoretools.GraceContainer(
                ...     [Note("ds'16")],
                ...     kind='after'
                ...     )
                >>> attach(after_grace, staff[1])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    c'8
                    \grace {
                        cs'16
                    }
                    \afterGrace
                    d'8
                    {
                        ds'16
                    }
                    e'8
                    f'8
                }

            ::

                >>> inspect_(staff[1]).get_grace_containers()
                (GraceContainer("cs'16"), GraceContainer("ds'16"))

        ..  container:: example

            **Example 2.** Get only (proper) grace containers attached to note:

            ::

                >>> inspect_(staff[1]).get_grace_containers(kind='grace')
                (GraceContainer("cs'16"),)

        ..  container:: example

            **Example 3.** Get only after grace containers attached to note:

            ::

                >>> inspect_(staff[1]).get_grace_containers(kind='after')
                (GraceContainer("ds'16"),)

        Set `kind` to ``'grace'``, ``'after'`` or none.

        Returns tuple.
        '''
        return self._client._get_grace_containers(
            kind=kind,
            )

    def get_indicator(
        self,
        prototype=None,
        default=None,
        unwrap=True,
        ):
        r'''Gets indicator of `prototype` attached to client.

        Raises exception when more than one indicator of `prototype` attach to
        client.

        Returns default when no indicator of `prototype` attaches to client.

        Returns indicator or default.
        '''
        indicators = self._client._get_indicators(
                prototype=prototype,
                unwrap=unwrap,
                )
        if not indicators:
            return default
        elif len(indicators) == 1:
            return list(indicators)[0]
        else:
            message = 'multiple indicators attached to client.'
            raise Exception(message)

    def get_indicators(
        self,
        prototype=None,
        unwrap=True,
        ):
        r'''Get all indicators matching `prototype` attached
        to client.

        Returns tuple.
        '''
        return self._client._get_indicators(
            prototype=prototype,
            unwrap=unwrap,
            )

    def get_leaf(self, n=0):
        r'''Gets leaf `n`.

        ..  container:: example

            Example score:

            ::

                >>> staff = Staff()
                >>> staff.append(Voice("c'8 d'8 e'8 f'8"))
                >>> staff.append(Voice("g'8 a'8 b'8 c''8"))
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    \new Voice {
                        c'8
                        d'8
                        e'8
                        f'8
                    }
                    \new Voice {
                        g'8
                        a'8
                        b'8
                        c''8
                    }
                }

        ..  container:: example

            **Example 1.** Gets leaf `n` **from** client of inspection agent
            when client of inspection agent is a leaf.

            With positive indices:

            ::

                >>> first_leaf = staff[0][0]
                >>> first_leaf
                Note("c'8")

            ::

                >>> for n in range(8):
                ...     print(n, inspect_(first_leaf).get_leaf(n))
                ...
                0 c'8
                1 d'8
                2 e'8
                3 f'8
                4 None
                5 None
                6 None
                7 None

            With negative indices:

            ::

                >>> last_leaf = staff[0][-1]
                >>> last_leaf
                Note("f'8")

            ::

                >>> for n in range(0, -8, -1):
                ...     print(n, inspect_(last_leaf).get_leaf(n))
                ...
                0 f'8
                -1 e'8
                -2 d'8
                -3 c'8
                -4 None
                -5 None
                -6 None
                -7 None

        ..  container:: example

            **Example 2.** Gets leaf `n` **in** client of inspection agent
            when client of inspection agent is a container.

            With positive indices:

            ::

                >>> first_voice = staff[0]
                >>> first_voice
                Voice("c'8 d'8 e'8 f'8")

            ::

                >>> for n in range(8):
                ...     print(n, inspect_(first_voice).get_leaf(n))
                ...
                0 c'8
                1 d'8
                2 e'8
                3 f'8
                4 None
                5 None
                6 None
                7 None

            With negative indices:

            ::

                >>> first_voice = staff[0]
                >>> first_voice
                Voice("c'8 d'8 e'8 f'8")

            ::

                >>> for n in range(-1, -9, -1):
                ...     print(n, inspect_(first_voice).get_leaf(n))
                ...
                -1 f'8
                -2 e'8
                -3 d'8
                -4 c'8
                -5 None
                -6 None
                -7 None
                -8 None

        Returns leaf or none.
        '''
        from abjad.tools import scoretools
        if isinstance(self._client, scoretools.Leaf):
            return self._client._get_leaf(n=n)
        if 0 <= n:
            leaves = iterate(self._client).by_leaf(start=0, stop=n+1)
            leaves = list(leaves)
            if len(leaves) < n + 1:
                return
            leaf = leaves[n]
            return leaf
        else:
            leaves = iterate(self._client).by_leaf(
                start=0,
                stop=abs(n),
                reverse=True,
                )
            leaves = list(leaves)
            if len(leaves) < abs(n):
                return
            leaf = leaves[abs(n)-1]
            return leaf

    def get_lineage(self):
        r'''Gets lineage of client.

        Returns lineage.
        '''
        return self._client._get_lineage()

    def get_logical_tie(self):
        r'''Gets logical tie that governs leaf.

        Returns logical tie.
        '''
        return self._client._get_logical_tie()

    def get_markup(
        self,
        direction=None,
        ):
        r'''Gets all markup attached to client.

        Returns tuple.
        '''
        return self._client._get_markup(
            direction=direction,
            )

    def get_parentage(
        self,
        include_self=True,
        with_grace_notes=False,
        ):
        r'''Gets parentage of client.

        .. container:: example

            **Example 1.** Gets parentage without grace notes:

            ::

                >>> voice = Voice("c'4 d'4 e'4 f'4")
                >>> grace_notes = [Note("c'16"), Note("d'16")]
                >>> grace_container = scoretools.GraceContainer(
                ...     grace_notes,
                ...     kind='grace',
                ...     )
                >>> attach(grace_container, voice[1])
                >>> show(voice) # doctest: +SKIP

            ..  doctest::

                >>> f(voice)
                \new Voice {
                    c'4
                    \grace {
                        c'16
                        d'16
                    }
                    d'4
                    e'4
                    f'4
                }

            ::

                >>> inspect_(grace_notes[0]).get_parentage()
                Parentage([Note("c'16"), GraceContainer("c'16 d'16")])

        .. container:: example

            **Example 2.** Gets parentage with grace notes:

            ::

                >>> voice = Voice("c'4 d'4 e'4 f'4")
                >>> grace_notes = [Note("c'16"), Note("d'16")]
                >>> grace_container = scoretools.GraceContainer(
                ...     grace_notes,
                ...     kind='grace',
                ...     )
                >>> attach(grace_container, voice[1])
                >>> show(voice) # doctest: +SKIP

            ..  doctest::

                >>> f(voice)
                \new Voice {
                    c'4
                    \grace {
                        c'16
                        d'16
                    }
                    d'4
                    e'4
                    f'4
                }

            ::

                >>> inspector = inspect_(grace_notes[0])
                >>> inspector.get_parentage(with_grace_notes=True)
                Parentage([Note("c'16"), GraceContainer("c'16 d'16"), Note("d'4"), Voice("c'4 d'4 e'4 f'4")])

        Returns parentage.
        '''
        return self._client._get_parentage(
            include_self=include_self,
            with_grace_notes=with_grace_notes,
            )

    def get_sounding_pitch(self):
        r'''Gets sounding pitch of client.

        ..  container:: example

            ::

                >>> staff = Staff("d''8 e''8 f''8 g''8")
                >>> piccolo = instrumenttools.Piccolo()
                >>> attach(piccolo, staff)
                >>> instrumenttools.transpose_from_sounding_pitch_to_written_pitch(
                ...     staff)
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    \set Staff.instrumentName = \markup { Piccolo }
                    \set Staff.shortInstrumentName = \markup { Picc. }
                    d'8
                    e'8
                    f'8
                    g'8
                }
                >>> inspect_(staff[0]).get_sounding_pitch()
                NamedPitch("d''")

        Returns named pitch.
        '''
        return self._client._get_sounding_pitch()

    def get_sounding_pitches(self):
        r"""Gets sounding pitches of client.

        ..  container:: example

            ::

                >>> staff = Staff("<c''' e'''>4 <d''' fs'''>4")
                >>> glockenspiel = instrumenttools.Glockenspiel()
                >>> attach(glockenspiel, staff)
                >>> instrumenttools.transpose_from_sounding_pitch_to_written_pitch(
                ...     staff)
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    \set Staff.instrumentName = \markup { Glockenspiel }
                    \set Staff.shortInstrumentName = \markup { Gkspl. }
                    <c' e'>4
                    <d' fs'>4
                }

            ::

                >>> inspect_(staff[0]).get_sounding_pitches()
                (NamedPitch("c'''"), NamedPitch("e'''"))

        Returns tuple.
        """
        return self._client._get_sounding_pitches()

    def get_spanner(
        self,
        prototype=None,
        default=None,
        in_parentage=False,
        ):
        r'''Gets spanner of `prototype` attached to client.

        Raises exception when more than one spanner of `prototype` attaches to
        client.

        Returns `default` when no spanner of `prototype` attaches to client.

        Returns spanner or default.
        '''
        spanners = self._client._get_spanners(
            prototype=prototype,
            in_parentage=in_parentage,
            )
        if not spanners:
            return default
        elif len(spanners) == 1:
            return list(spanners)[0]
        else:
            message = 'multiple spanners attached to client.'
            raise Exception(message)

    def get_spanners(
        self,
        prototype=None,
        in_parentage=False,
        ):
        r'''Gets spanners attached to client.

        Returns set.
        '''
        return self._client._get_spanners(
            prototype=prototype,
            in_parentage=in_parentage,
            )

    def get_timespan(self, in_seconds=False):
        r'''Gets timespan of client.

        ..  container:: example

            **Example.** Gets timespan of grace notes:

            ::

                >>> voice = Voice("c'8 [ d'8 e'8 f'8 ]")
                >>> grace_notes = [Note("c'16"), Note("d'16")]
                >>> grace = scoretools.GraceContainer(
                ...     grace_notes,
                ...     kind='grace',
                ...     )
                >>> attach(grace, voice[1])
                >>> after_grace_notes = [Note("e'16"), Note("f'16")]
                >>> after_grace = scoretools.GraceContainer(
                ...     after_grace_notes,
                ...     kind='after')
                >>> attach(after_grace, voice[1])
                >>> show(voice) # doctest: +SKIP

            ..  doctest::

                >>> f(voice)
                \new Voice {
                    c'8 [
                    \grace {
                        c'16
                        d'16
                    }
                    \afterGrace
                    d'8
                    {
                        e'16
                        f'16
                    }
                    e'8
                    f'8 ]
                }

            ::

                >>> for leaf in iterate(voice).by_leaf(with_grace_notes=True):
                ...     timespan = inspect_(leaf).get_timespan()
                ...     print(str(leaf) + ':')
                ...     print(format(timespan, 'storage'))
                c'8:
                timespantools.Timespan(
                    start_offset=durationtools.Offset(0, 1),
                    stop_offset=durationtools.Offset(1, 8),
                    )
                c'16:
                timespantools.Timespan(
                    start_offset=durationtools.Offset(
                        (1, 8),
                        grace_displacement=durationtools.Duration(-1, 8),
                        ),
                    stop_offset=durationtools.Offset(
                        (1, 8),
                        grace_displacement=durationtools.Duration(-1, 16),
                        ),
                    )
                d'16:
                timespantools.Timespan(
                    start_offset=durationtools.Offset(
                        (1, 8),
                        grace_displacement=durationtools.Duration(-1, 16),
                        ),
                    stop_offset=durationtools.Offset(1, 8),
                    )
                d'8:
                timespantools.Timespan(
                    start_offset=durationtools.Offset(1, 8),
                    stop_offset=durationtools.Offset(1, 4),
                    )
                e'16:
                timespantools.Timespan(
                    start_offset=durationtools.Offset(
                        (1, 4),
                        grace_displacement=durationtools.Duration(-1, 8),
                        ),
                    stop_offset=durationtools.Offset(
                        (1, 4),
                        grace_displacement=durationtools.Duration(-1, 16),
                        ),
                    )
                f'16:
                timespantools.Timespan(
                    start_offset=durationtools.Offset(
                        (1, 4),
                        grace_displacement=durationtools.Duration(-1, 16),
                        ),
                    stop_offset=durationtools.Offset(1, 4),
                    )
                e'8:
                timespantools.Timespan(
                    start_offset=durationtools.Offset(1, 4),
                    stop_offset=durationtools.Offset(3, 8),
                    )
                f'8:
                timespantools.Timespan(
                    start_offset=durationtools.Offset(3, 8),
                    stop_offset=durationtools.Offset(1, 2),
                    )

        Returns timespan.
        '''
        return self._client._get_timespan(
            in_seconds=in_seconds,
            )

    def get_vertical_moment(
        self,
        governor=None,
        ):
        r'''Gets vertical moment starting with client.

        Returns vertical moment.
        '''
        return self._client._get_vertical_moment(
            governor=governor,
            )

    def get_vertical_moment_at(
        self,
        offset,
        ):
        r'''Gets vertical moment at `offset`.

        Returns vertical moment.
        '''
        return self._client._get_vertical_moment_at(
            offset,
            )

    def has_effective_indicator(self, prototype=None):
        r'''Is true when indicator that matches `prototype` is
        in effect for client. Otherwise false.

        Returns true or false.
        '''
        return self._client._has_effective_indicator(prototype=prototype)

    def has_indicator(self, prototype=None):
        r'''Is true when client has one or more
        indicators that match `prototype`. Otherwise false.

        Returns true or false.
        '''
        return self._client._has_indicator(prototype=prototype)

    def has_spanner(
        self,
        prototype=None,
        in_parentage=False,
        ):
        r'''Is true when client has one or more
        spanners that match `prototype`. Otherwise false.

        Returns true or false.
        '''
        return self._client._has_spanner(
            prototype=prototype,
            in_parentage=in_parentage,
            )

    def is_bar_line_crossing(self):
        r'''Is true when client crosses bar line.
        Otherwise false.

        ..  container:: example

            ::

                >>> staff = Staff("c'4 d'4 e'4")
                >>> time_signature = TimeSignature((3, 8))
                >>> attach(time_signature, staff)
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    \time 3/8
                    c'4
                    d'4
                    e'4
                }

            ::

                >>> for note in staff:
                ...     result = inspect_(note).is_bar_line_crossing()
                ...     print(note, result)
                ...
                c'4 False
                d'4 True
                e'4 False

        Returns true or false.
        '''
        from abjad.tools import indicatortools
        time_signature = self._client._get_effective(
            indicatortools.TimeSignature)
        if time_signature is None:
            time_signature_duration = durationtools.Duration(4, 4)
        else:
            time_signature_duration = time_signature.duration
        partial = getattr(time_signature, 'partial', 0)
        partial = partial or 0
        start_offset = self._client._get_timespan().start_offset
        shifted_start = start_offset - partial
        shifted_start %= time_signature_duration
        stop_offset = self._client._get_duration() + shifted_start
        if time_signature_duration < stop_offset:
            return True
        return False

    def is_well_formed(self):
        r'''Is true when client is well-formed.
        Otherwise false.

        Returns false.
        '''
        from abjad.tools import systemtools
        manager = systemtools.WellformednessManager()
        for violators, total, check_name in manager(self._client):
            if violators:
                return False
        return True

    def report_modifications(self):
        r'''Reports modifications of client.

        ..  container:: example

            Report modifications of container in selection:

            ::

                >>> container = Container("c'8 d'8 e'8 f'8")
                >>> override(container).note_head.color = 'red'
                >>> override(container).note_head.style = 'harmonic'
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> f(container)
                {
                    \override NoteHead.color = #red
                    \override NoteHead.style = #'harmonic
                    c'8
                    d'8
                    e'8
                    f'8
                    \revert NoteHead.color
                    \revert NoteHead.style
                }

            ::

                >>> report = inspect_(container).report_modifications()

            ::

                >>> print(report)
                {
                    \override NoteHead.color = #red
                    \override NoteHead.style = #'harmonic
                    %%% 4 components omitted %%%
                    \revert NoteHead.color
                    \revert NoteHead.style
                }

        Returns string.
        '''
        from abjad.tools import scoretools
        from abjad.tools import systemtools
        client = self._client
        bundle = systemtools.LilyPondFormatManager.bundle_format_contributions(
            client)
        result = []
        result.extend(client._get_format_contributions_for_slot(
            'before', bundle))
        result.extend(client._get_format_contributions_for_slot(
            'open brackets', bundle))
        result.extend(client._get_format_contributions_for_slot(
            'opening', bundle))
        result.append('    %%%%%% %s components omitted %%%%%%' % len(client))
        result.extend(client._get_format_contributions_for_slot(
            'closing', bundle))
        result.extend(client._get_format_contributions_for_slot(
            'close brackets', bundle))
        result.extend(client._get_format_contributions_for_slot(
            'after', bundle))
        result = '\n'.join(result)
        return result

    def tabulate_well_formedness_violations(self):
        r'''Tabulates well-formedness violations in client.

        ..  container:: example

            ::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> staff[1].written_duration = Duration(1, 4)
                >>> beam = spannertools.Beam()
                >>> attach(beam, staff[:])

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    c'8 [
                    d'4
                    e'8
                    f'8 ]
                }

            ::

                >>> result = inspect_(staff).tabulate_well_formedness_violations()

            ::

                >>> print(result)
                1 / 4 beamed quarter notes
                0 / 1 conflicting clefs
                0 / 1 discontiguous spanners
                0 / 5 duplicate ids
                0 / 1 empty containers
                0 / 0 intermarked hairpins
                0 / 0 misdurated measures
                0 / 0 misfilled measures
                0 / 0 mismatched enchained hairpins
                0 / 0 mispitched ties
                0 / 4 misrepresented flags
                0 / 5 missing parents
                0 / 0 nested measures
                0 / 1 overlapping beams
                0 / 0 overlapping glissandi
                0 / 0 overlapping hairpins
                0 / 0 overlapping octavation spanners
                0 / 0 overlapping ties
                0 / 0 short hairpins
                0 / 0 tied rests

            Beamed quarter notes are not well formed.

        Returns string.
        '''
        from abjad.tools import systemtools
        manager = systemtools.WellformednessManager()
        triples = manager(self._client)
        strings = []
        for violators, total, check_name in triples:
            violator_count = len(violators)
            string = '{} /\t{} {}'
            check_name = check_name.replace('check_', '')
            check_name = check_name.replace('_', ' ')
            string = string.format(violator_count, total, check_name)
            strings.append(string)
        return '\n'.join(strings)

    ### PUBLIC PROPERTIES ###

    @property
    def client(self):
        r'''Client of inspection agent.

        Returns component.
        '''
        return self._client



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_Beam__fracture_01():
    r'''This test shows that fracturing beyond the first leaf
    effectively does nothing except to replace an existing
    spanner with an identical new spanner.
    '''

    staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    beam = Beam()
    attach(beam, staff[:4])
    beam._fracture(0, direction=Left)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [
            d'8
            e'8
            f'8 ]
            g'8
            a'8
            b'8
            c''8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_spannertools_Beam__fracture_02():

    staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    beam = Beam()
    attach(beam, staff[:4])
    beam._fracture(1, direction=Left)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [ ]
            d'8 [
            e'8
            f'8 ]
            g'8
            a'8
            b'8
            c''8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_spannertools_Beam__fracture_03():
    r'''This test shows that fracurting beyond the last leaf
    effectively does nothing except to replace an existing
    spanner with an identical new spanner.
    '''

    staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    beam = Beam()
    attach(beam, staff[:4])
    beam._fracture(-1, direction=Right)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [
            d'8
            e'8
            f'8 ]
            g'8
            a'8
            b'8
            c''8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_spannertools_Beam__fracture_04():

    staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    beam = Beam()
    attach(beam, staff[:4])
    beam._fracture(1, direction=Right)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [
            d'8 ]
            e'8 [
            f'8 ]
            g'8
            a'8
            b'8
            c''8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_spannertools_Beam__fracture_05():
    r'''Fracture both sides of leaf.
    '''

    staff = Staff("c'8 cs'8 d'8 ef'8 e'8 f'8 fs'8 g'8")
    beam = Beam()
    attach(beam, staff[:5])
    beam._fracture(2, direction=None)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [
            cs'8 ]
            d'8 [ ]
            ef'8 [
            e'8 ]
            f'8
            fs'8
            g'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_spannertools_Beam__fracture_06():
    r'''Fracture both sides of first leaf in spanner.
    '''

    staff = Staff("c'8 cs'8 d'8 ef'8 e'8 f'8 fs'8 g'8")
    beam = Beam()
    attach(beam, staff[:5])
    beam._fracture(0, direction=None)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [ ]
            cs'8 [
            d'8
            ef'8
            e'8 ]
            f'8
            fs'8
            g'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_spannertools_Beam__fracture_07():
    r'''Fracture both sides of last leaf in spanner.
    '''

    staff = Staff("c'8 cs'8 d'8 ef'8 e'8 f'8 fs'8 g'8")
    beam = Beam()
    attach(beam, staff[:5])
    beam._fracture(4, direction=None)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [
            cs'8
            d'8
            ef'8 ]
            e'8 [ ]
            f'8
            fs'8
            g'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_spannertools_Beam__fracture_08():
    r'''Fracture both sides of leaf with negative index.
    '''

    staff = Staff("c'8 cs'8 d'8 ef'8 e'8 f'8 fs'8 g'8")
    beam = Beam()
    attach(beam, staff[:5])
    beam._fracture(-1, direction=None)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [
            cs'8
            d'8
            ef'8 ]
            e'8 [ ]
            f'8
            fs'8
            g'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_spannertools_Beam__fracture_09():
    r'''Fracture container spanner to the right of index 1.
    '''

    staff = Staff(
        r'''
        {
            c'8
            cs'8
            d'8
            ef'8
        }
        {
            e'8
            f'8
            fs'8
            g'8
        }
        {
            af'8
            a'8
            bf'8
            b'8
        }
        '''
        )

    leaves = select(staff).by_leaf()
    beam = Beam()
    attach(beam, leaves)
    original, left, right = beam._fracture(7, direction=Right)

    assert len(original) == 12
    assert len(left) == 8
    assert len(right) == 4

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                c'8 [
                cs'8
                d'8
                ef'8
            }
            {
                e'8
                f'8
                fs'8
                g'8 ]
            }
            {
                af'8 [
                a'8
                bf'8
                b'8 ]
            }
        }
        '''
        )



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_Spanner_format_01():
    r'''Base Spanner class makes no format-time contributions.
    However, base spanner causes no explosions at format-time, either.
    '''

    class MockSpanner(spannertools.Spanner):

        def __init__(self, components=None):
            spannertools.Spanner.__init__(self, components)

    staff = Staff("c'8 d'8 e'8 f'8")
    spanner = MockSpanner()
    attach(spanner, staff[:])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8
            d'8
            e'8
            f'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()



# -*- coding: utf-8 -*-
from abjad import *


# TODO: move to test_Measure___init__.py

def test_scoretools_Measure_in_place_apply_01():

    voice = Voice([Note(n, (1, 8)) for n in range(8)])
    leaves_before = voice[:]
    Measure((4, 8), voice[0:4])
    leaves_after = voice[:]

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 4/8
                c'8
                cs'8
                d'8
                ef'8
            }
            e'8
            f'8
            fs'8
            g'8
        }
        '''
        )


def test_scoretools_Measure_in_place_apply_02():

    staff = Staff([Note(n, (1, 8)) for n in range(8)])
    leaves_before = staff[:]
    Measure((4, 8), staff[0:4])
    leaves_after = staff[:]

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 4/8
                c'8
                cs'8
                d'8
                ef'8
            }
            e'8
            f'8
            fs'8
            g'8
        }
        '''
        )


def test_scoretools_Measure_in_place_apply_03():

    staff = Staff([Note(n, (1, 1)) for n in range(4)])
    leaves_before = staff[:]
    Measure((1, 1), staff[0:1])
    leaves_after = staff[:]

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 1/1
                c'1
            }
            cs'1
            d'1
            ef'1
        }
        '''
        )


def test_scoretools_Measure_in_place_apply_04():

    staff = Staff([Note(n, (1, 1)) for n in range(4)])
    Measure((1, 1), staff[:1])
    Measure((1, 1), staff[1:2])
    Measure((1, 1), staff[2:3])
    Measure((1, 1), staff[3:])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 1/1
                c'1
            }
            {
                cs'1
            }
            {
                d'1
            }
            {
                ef'1
            }
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


# -*- coding: utf-8 -*-
import copy
from abjad.tools import datastructuretools
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import patterntools
from abjad.tools import scoretools
from abjad.tools import selectiontools
from abjad.tools import sequencetools
from abjad.tools import spannertools
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import detach
from abjad.tools.topleveltools import inspect_
from abjad.tools.topleveltools import iterate
from abjad.tools.topleveltools import mutate


class RhythmMaker(AbjadValueObject):
    '''Rhythm-maker.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Rhythm-makers'

    __slots__ = (
        '_beam_specifier',
        '_logical_tie_masks',
        '_division_masks',
        '_duration_spelling_specifier',
        '_rotation',
        '_tie_specifier',
        '_tuplet_spelling_specifier',
        )

    _publish_storage_format = True

    ### INITIALIZER ###

    def __init__(
        self,
        beam_specifier=None,
        logical_tie_masks=None,
        division_masks=None,
        duration_spelling_specifier=None,
        tie_specifier=None,
        tuplet_spelling_specifier=None,
        ):
        from abjad.tools import rhythmmakertools
        prototype = (rhythmmakertools.BeamSpecifier, type(None))
        assert isinstance(beam_specifier, prototype)
        self._beam_specifier = beam_specifier
        logical_tie_masks = self._prepare_masks(logical_tie_masks)
        self._logical_tie_masks = logical_tie_masks
        prototype = (rhythmmakertools.DurationSpellingSpecifier, type(None))
        self._duration_spelling_specifier = duration_spelling_specifier
        assert isinstance(duration_spelling_specifier, prototype)
        division_masks = self._prepare_masks(division_masks)
        self._division_masks = division_masks
        prototype = (rhythmmakertools.TieSpecifier, type(None))
        assert isinstance(tie_specifier, prototype)
        self._tie_specifier = tie_specifier
        prototype = (rhythmmakertools.TupletSpellingSpecifier, type(None))
        assert isinstance(tuplet_spelling_specifier, prototype)
        self._tuplet_spelling_specifier = tuplet_spelling_specifier

    ### SPECIAL METHODS ###

    def __call__(self, divisions, rotation=None):
        r'''Calls rhythm-maker.

        Returns selections.
        '''
        self._rotation = rotation
        divisions = self._coerce_divisions(divisions)
        selections = self._make_music(divisions, rotation)
        selections = self._apply_specifiers(selections, divisions)
        self._check_well_formedness(selections)
        return selections

    def __illustrate__(self, divisions=((3, 8), (4, 8), (3, 16), (4, 16))):
        r'''Illustrates rhythm-maker.

        Returns LilyPond file.
        '''
        import abjad
        selections = self(divisions)
        lilypond_file = abjad.rhythmmakertools.make_lilypond_file(
            selections,
            divisions,
            )
        return lilypond_file

    ### PRIVATE METHODS ###

    @staticmethod
    def _all_are_tuplets_or_all_are_leaf_selections(argument):
        if all(isinstance(x, scoretools.Tuplet) for x in argument):
            return True
        elif all(RhythmMaker._is_leaf_selection(x) for x in argument):
            return True
        else:
            return False

    def _apply_division_masks(self, selections, rotation=None):
        from abjad.tools import rhythmmakertools
        if not self.division_masks:
            return selections
        new_selections = []
        duration_spelling_specifier = self._get_duration_spelling_specifier()
        decrease_durations_monotonically = \
            duration_spelling_specifier.decrease_durations_monotonically
        forbidden_written_duration = \
            duration_spelling_specifier.forbidden_written_duration
        tie_specifier = self._get_tie_specifier()
        length = len(selections)
        division_masks = self.division_masks
        for i, selection in enumerate(selections):
            matching_division_mask = division_masks.get_matching_pattern(
                i,
                length,
                rotation=rotation,
                )
            if not matching_division_mask:
                new_selections.append(selection)
                continue
            duration = selection.get_duration()
            if isinstance(
                matching_division_mask,
                rhythmmakertools.SustainMask,
                ):
                new_selection = scoretools.make_leaves(
                    [0],
                    [duration],
                    decrease_durations_monotonically=\
                        decrease_durations_monotonically,
                    forbidden_written_duration=forbidden_written_duration,
                    use_messiaen_style_ties=\
                        tie_specifier.use_messiaen_style_ties,
                    )
            else:
                use_multimeasure_rests = getattr(
                    matching_division_mask,
                    'use_multimeasure_rests',
                    False,
                    )
                new_selection = scoretools.make_leaves(
                    [None],
                    [duration],
                    decrease_durations_monotonically=\
                        decrease_durations_monotonically,
                    forbidden_written_duration=forbidden_written_duration,
                    use_multimeasure_rests=use_multimeasure_rests,
                    )
            for component in iterate(selection).by_class():
                detach(spannertools.Tie, component)
            new_selections.append(new_selection)
        return new_selections

    def _apply_logical_tie_masks(self, selections):
        from abjad.tools import rhythmmakertools
        if self.logical_tie_masks is None:
            return selections
        # wrap every selection in a temporary container;
        # this allows the call to mutate().replace() to work
        containers = []
        for selection in selections:
            container = scoretools.Container(selection)
            attach('temporary container', container)
            containers.append(container)
        logical_ties = iterate(selections).by_logical_tie()
        logical_ties = list(logical_ties)
        total_logical_ties = len(logical_ties)
        for index, logical_tie in enumerate(logical_ties[:]):
            matching_mask = self.logical_tie_masks.get_matching_pattern(
                index,
                total_logical_ties,
                )
            if not isinstance(matching_mask, rhythmmakertools.SilenceMask):
                continue
            if isinstance(logical_tie.head, scoretools.Rest):
                continue
            for leaf in logical_tie:
                rest = scoretools.Rest(leaf.written_duration)
                inspector = inspect_(leaf)
                if inspector.has_indicator(durationtools.Multiplier):
                    multiplier = inspector.get_indicator(
                        durationtools.Multiplier,
                        )
                    multiplier = durationtools.Multiplier(multiplier)
                    attach(multiplier, rest)
                mutate(leaf).replace([rest])
                detach(spannertools.Tie, rest)
        # remove every temporary container and recreate selections
        new_selections = []
        for container in containers:
            inspector = inspect_(container)
            assert inspector.get_indicator(str) == 'temporary container'
            new_selection = mutate(container).eject_contents()
            new_selections.append(new_selection)
        return new_selections

    def _apply_specifiers(self, selections, divisions=None):
        selections = self._apply_tuplet_spelling_specifier(
            selections,
            divisions,
            )
        self._apply_tie_specifier(selections)
        selections = self._apply_logical_tie_masks(selections)
        self._validate_selections(selections)
        self._validate_tuplets(selections)
        return selections

    def _apply_tie_specifier(self, selections):
        tie_specifier = self._get_tie_specifier()
        tie_specifier(selections)


    def _apply_tuplet_spelling_specifier(self, selections, divisions):
        # TODO: migrate functionality to TupletSpellingSpecifier.__call__()
        tuplet_spelling_specifier = self._get_tuplet_spelling_specifier()
        tuplet_spelling_specifier._do_simplify_redundant_tuplets(selections)
        selections = self._rewrite_rest_filled_tuplets(selections)
        selections = self._flatten_trivial_tuplets(selections)
        tuplet_spelling_specifier._apply_preferred_denominator(
            selections,
            divisions,
            )
        return selections

    def _check_well_formedness(self, selections):
        for component in iterate(selections).by_class():
            inspector = inspect_(component)
            if not inspector.is_well_formed():
                report = inspector.tabulate_well_formedness_violations()
                report = repr(component) + '\n' + report
                raise Exception(report)

    @staticmethod
    def _coerce_divisions(divisions):
        divisions_ = []
        for division in divisions:
            if isinstance(division, mathtools.NonreducedFraction):
                divisions_.append(division)
            else:
                division = mathtools.NonreducedFraction(division)
                divisions_.append(division)
        divisions = divisions_
        prototype = mathtools.NonreducedFraction
        assert all( isinstance(_, prototype) for _ in divisions)
        return divisions

    def _flatten_trivial_tuplets(self, selections):
        tuplet_spelling_specifier = self._get_tuplet_spelling_specifier()
        if not tuplet_spelling_specifier.flatten_trivial_tuplets:
            return selections
        new_selections = []
        for selection in selections:
            new_selection = []
            for component in selection:
                if not (isinstance(component, scoretools.Tuplet) and
                    component.is_trivial):
                    new_selection.append(component)
                    continue
                spanners = inspect_(component).get_spanners()
                contents = component[:]
                for spanner in spanners:
                    new_spanner = copy.copy(spanner)
                    attach(new_spanner, contents)
                new_selection.extend(contents)
                del(component[:])
            new_selection = selectiontools.Selection(new_selection)
            new_selections.append(new_selection)
        return new_selections

    def _get_beam_specifier(self):
        from abjad.tools import rhythmmakertools
        if self.beam_specifier is not None:
            return self.beam_specifier
        return rhythmmakertools.BeamSpecifier()

    def _get_duration_spelling_specifier(self):
        from abjad.tools import rhythmmakertools
        if self.duration_spelling_specifier is not None:
            return self.duration_spelling_specifier
        return rhythmmakertools.DurationSpellingSpecifier()

    def _get_tie_specifier(self):
        from abjad.tools import rhythmmakertools
        if self.tie_specifier is not None:
            return self.tie_specifier
        return rhythmmakertools.TieSpecifier()

    def _get_tuplet_spelling_specifier(self):
        from abjad.tools import rhythmmakertools
        if self.tuplet_spelling_specifier is not None:
            return self.tuplet_spelling_specifier
        return rhythmmakertools.TupletSpellingSpecifier()

    @staticmethod
    def _is_leaf_selection(argument):
        if isinstance(argument, selectiontools.Selection):
            return all(isinstance(x, scoretools.Leaf) for x in argument)
        return False

    @staticmethod
    def _is_sign_tuple(argument):
        if isinstance(argument, tuple):
            prototype = (-1, 0, 1)
            return all(_ in prototype for _ in argument)
        return False

    @staticmethod
    def _make_cyclic_tuple_generator(iterable):
        cyclic_tuple = datastructuretools.CyclicTuple(iterable)
        i = 0
        while True:
            yield cyclic_tuple[i]
            i += 1

    def _make_secondary_divisions(
        self,
        divisions,
        split_divisions_by_counts,
        ):
        if not split_divisions_by_counts:
            return divisions[:]
        numerators = [
            division.numerator
            for division in divisions
            ]
        secondary_numerators = sequencetools.Sequence(numerators)
        secondary_numerators = secondary_numerators.split(
            split_divisions_by_counts,
            cyclic=True,
            overhang=True,
            )
        secondary_numerators = sequencetools.Sequence(secondary_numerators)
        secondary_numerators = secondary_numerators.flatten()
        denominator = divisions[0].denominator
        secondary_divisions = [
            (n, denominator)
            for n in secondary_numerators
            ]
        return secondary_divisions

    def _make_tuplets(self, divisions, leaf_lists):
        assert len(divisions) == len(leaf_lists)
        tuplets = []
        for division, leaf_list in zip(divisions, leaf_lists):
            tuplet = scoretools.FixedDurationTuplet(division, leaf_list)
            tuplets.append(tuplet)
        return tuplets

    def _none_to_trivial_helper(self, argument):
        if argument is None:
            argument = self._trivial_helper
        assert callable(argument)
        return argument

    @staticmethod
    def _prepare_masks(masks):
        from abjad.tools import rhythmmakertools
        prototype = (
            rhythmmakertools.SilenceMask,
            rhythmmakertools.SustainMask,
            )
        if masks is None:
            return
        if isinstance(masks, patterntools.Pattern):
            masks = (masks,)
        if isinstance(masks, prototype):
            masks = (masks,)
        masks = patterntools.PatternList(
            items=masks,
            )
        return masks

    @staticmethod
    def _reverse_tuple(argument):
        if argument is not None:
            return tuple(reversed(argument))

    def _rewrite_rest_filled_tuplets(self, selections):
        tuplet_spelling_specifier = self._get_tuplet_spelling_specifier()
        if not tuplet_spelling_specifier.rewrite_rest_filled_tuplets:
            return selections
        new_selections = []
        for selection in selections:
            new_selection = []
            for component in selection:
                if not (isinstance(component, scoretools.Tuplet) and
                    component._is_rest_filled):
                    new_selection.append(component)
                    continue
                duration = inspect_(component).get_duration()
                new_rests = scoretools.make_rests([duration])
                mutate(component[:]).replace(new_rests)
                new_selection.append(component)
            new_selection = selectiontools.Selection(new_selection)
            new_selections.append(new_selection)
        return new_selections

    @staticmethod
    def _rotate_tuple(argument, n):
        if argument is not None:
            return tuple(sequencetools.Sequence(argument).rotate(n=n))

    def _scale_taleas(self, divisions, talea_denominator, taleas):
        talea_denominator = talea_denominator or 1
        dummy_division = (1, talea_denominator)
        divisions.append(dummy_division)
        Duration = durationtools.Duration
        divisions = Duration.durations_to_nonreduced_fractions(divisions)
        dummy_division = divisions.pop()
        lcd = dummy_division.denominator
        multiplier = lcd / talea_denominator
        assert mathtools.is_integer_equivalent(multiplier), repr(multiplier)
        multiplier = int(multiplier)
        scaled_taleas = []
        for talea in taleas:
            talea = [multiplier * _ for _ in talea]
            talea = datastructuretools.CyclicTuple(talea)
            scaled_taleas.append(talea)
        result = [divisions, lcd]
        result.extend(scaled_taleas)
        return tuple(result)

    def _sequence_to_ellipsized_string(self, sequence):
        if not sequence:
            return '[]'
        if len(sequence) <= 4:
            result = ', '.join([str(x) for x in sequence])
        else:
            result = ', '.join([str(x) for x in sequence[:4]])
            result += ', ...'
        result = '[${}$]'.format(result)
        return result

    def _trivial_helper(self, sequence_, rotation):
        if isinstance(rotation, int) and len(sequence_):
            return sequencetools.Sequence(sequence_).rotate(n=rotation)
        return sequence_

    def _validate_selections(self, selections):
        assert isinstance(selections, list), repr(selections)
        assert len(selections), repr(selections)
        for selection in selections:
            assert isinstance(selection, selectiontools.Selection), selection

    def _validate_tuplets(self, selections):
        for tuplet in iterate(selections).by_class(scoretools.Tuplet):
            assert tuplet.multiplier.is_proper_tuplet_multiplier, repr(
                tuplet)
            assert len(tuplet), repr(tuplet)

    ### PUBLIC PROPERTIES ###

    @property
    def beam_specifier(self):
        r'''Gets beam specifier.

        Set to beam specifier or none.
        '''
        return self._beam_specifier

    @property
    def division_masks(self):
        r'''Gets division masks.

        Set to division masks or none.
        '''
        return self._division_masks

    @property
    def duration_spelling_specifier(self):
        r'''Gets duration spelling specifier.

        Set to duration spelling specifier or none.
        '''
        return self._duration_spelling_specifier

    @property
    def logical_tie_masks(self):
        r'''Gets logical tie masks.

        Set to patterns or none.

        Defaults to none.

        Returns patterns or none.
        '''
        return self._logical_tie_masks

    @property
    def tie_specifier(self):
        r'''Gets tie specifier.

        Set to tie specifier or none.
        '''
        return self._tie_specifier

    @property
    def tuplet_spelling_specifier(self):
        r'''Gets tuplet spelling specifier.

        Set to tuplet spelling specifier or none.
        '''
        return self._tuplet_spelling_specifier



# -*- coding: utf-8 -*-
from abjad import *


def test_quantizationtools_TerminalQEvent___eq___01():

    a = quantizationtools.TerminalQEvent(1000)
    b = quantizationtools.TerminalQEvent(1000)

    assert a == b


def test_quantizationtools_TerminalQEvent___eq___02():

    a = quantizationtools.TerminalQEvent(1000)
    b = quantizationtools.TerminalQEvent(9000)

    assert a != b


def test_quantizationtools_TerminalQEvent___eq___03():

    a = quantizationtools.TerminalQEvent(100)
    b = quantizationtools.PitchedQEvent(100, [0])
    c = quantizationtools.SilentQEvent(100)

    assert a != b
    assert a != c



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NumberedInterval___cmp___01():

    numbered_interval_1 = pitchtools.NumberedInterval(12)
    numbered_interval_2 = pitchtools.NumberedInterval(12)
    numbered_interval_3 = pitchtools.NumberedInterval(13)

    assert numbered_interval_1 == numbered_interval_1
    assert not numbered_interval_1 != numbered_interval_1
    assert not numbered_interval_1 < numbered_interval_1
    assert numbered_interval_1 <= numbered_interval_1
    assert not numbered_interval_1 > numbered_interval_1
    assert numbered_interval_1 >= numbered_interval_1

    assert numbered_interval_1 == numbered_interval_2
    assert not numbered_interval_1 != numbered_interval_2
    assert not numbered_interval_1 < numbered_interval_2
    assert numbered_interval_1 <= numbered_interval_2
    assert not numbered_interval_1 > numbered_interval_2
    assert numbered_interval_1 >= numbered_interval_2

    assert not numbered_interval_1 == numbered_interval_3
    assert numbered_interval_1 != numbered_interval_3
    assert numbered_interval_1 < numbered_interval_3
    assert numbered_interval_1 <= numbered_interval_3
    assert not numbered_interval_1 > numbered_interval_3
    assert not numbered_interval_1 >= numbered_interval_3



# -*- coding: utf-8 -*-


def to_bidirectional_lilypond_symbol(argument):
    r'''Changes `argument` to bidirectional LilyPond symbol.

    ..  container:: example

        ::

            >>> stringtools.to_tridirectional_lilypond_symbol(Up)
            '^'

    ..  container:: example

        ::

            >>> stringtools.to_tridirectional_lilypond_symbol(Down)
            '_'

    ..  container:: example

        ::

            >>> stringtools.to_tridirectional_lilypond_symbol(1)
            '^'

    ..  container:: example

        ::

            >>> stringtools.to_tridirectional_lilypond_symbol(-1)
            '_'

    Returns `argument` when `argument` is `'^'` or `'_'`.

    Returns string or none.
    '''
    lookup = {
        1: '^',
        -1: '_',
        Up: '^',
        Down: '_',
        'up': '^',
        'down': '_',
        '^': '^',
        '_': '_',
        }
    if argument in lookup:
        return lookup[argument]
    raise ValueError(argument)



# -*- coding: utf-8 -*-
import collections
from abjad.tools import pitchtools
from abjad.tools.abctools import AbjadValueObject


class PitchSelectorCallback(AbjadValueObject):
    r'''Pitch selector callback.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Callbacks'

    __slots__ = (
        '_pitches',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        pitches=None,
        ):
        if pitches is not None:
            if not isinstance(pitches, collections.Iterable):
                pitches = [pitches]
            pitches = pitchtools.PitchSet(
                items=pitches,
                item_class=pitchtools.NumberedPitch,
                )
        self._pitches = pitches

    ### SPECIAL METHODS ###

    def __call__(self, argument, rotation=None):
        r'''Iterates tuple `argument`.

        Returns tuple in which each item is a selection or component.
        '''
        if not self.pitches:
            return ()
        result = []
        for subexpr in argument:
            pitch_set = pitchtools.PitchSet.from_selection(
                subexpr,
                item_class=pitchtools.NumberedPitch,
                )
            if self.pitches.intersection(pitch_set):
                result.append(subexpr)
        return tuple(result)

    ### PUBLIC PROPERTIES ###

    @property
    def pitches(self):
        r'''Gets pitch set of pitch selector callback.

        Returns pitch set.
        '''
        return self._pitches



# -*- coding: utf-8 -*-
from abjad import *
import copy


def test_scoretools_Chord___deepcopy___01():
    r'''Chord deepchopies note-heads.
    '''

    chord_1 = Chord("<c' e' g'>4")
    chord_1.note_heads[0].tweak.color = 'red'
    chord_2 = copy.deepcopy(chord_1)

    assert format(chord_1) == stringtools.normalize(
        r'''
        <
            \tweak color #red
            c'
            e'
            g'
        >4
        '''
        )

    assert format(chord_2) == stringtools.normalize(
        r'''
        <
            \tweak color #red
            c'
            e'
            g'
        >4
        '''
        )

    assert chord_2.note_heads[0]._client is chord_2
    assert chord_2.note_heads[1]._client is chord_2
    assert chord_2.note_heads[2]._client is chord_2

    assert format(chord_1) == format(chord_2)
    assert chord_1 is not chord_2

    assert chord_1.note_heads[0] == chord_2.note_heads[0]
    assert chord_1.note_heads[1] == chord_2.note_heads[1]
    assert chord_1.note_heads[2] == chord_2.note_heads[2]

    assert chord_1.note_heads[0] is not chord_2.note_heads[0]
    assert chord_1.note_heads[1] is not chord_2.note_heads[1]
    assert chord_1.note_heads[2] is not chord_2.note_heads[2]



# -*- coding: utf-8 -*-
from abjad.tools.timespantools.Timespan import Timespan


class AnnotatedTimespan(Timespan):
    r'''An annotated timespan.

    ::

        >>> annotated_timespan = timespantools.AnnotatedTimespan(
        ...    annotation=['a', 'b', 'c'],
        ...    start_offset=Offset(1, 4),
        ...    stop_offset=Offset(7, 8),
        ...    )
        >>> print(format(annotated_timespan))
        timespantools.AnnotatedTimespan(
            start_offset=durationtools.Offset(1, 4),
            stop_offset=durationtools.Offset(7, 8),
            annotation=['a', 'b', 'c'],
            )

    Annotated timespans maintain their annotations duration mutation:

    ::

        >>> left, right = annotated_timespan.split_at_offset(Offset(1, 2))
        >>> left.annotation.append('foo')
        >>> print(format(right))
        timespantools.AnnotatedTimespan(
            start_offset=durationtools.Offset(1, 2),
            stop_offset=durationtools.Offset(7, 8),
            annotation=['a', 'b', 'c', 'foo'],
            )

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Timespans'

    __slots__ = (
        '_annotation',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        start_offset=NegativeInfinity,
        stop_offset=Infinity,
        annotation=None,
        ):
        Timespan.__init__(
            self,
            start_offset=start_offset,
            stop_offset=stop_offset,
            )
        self._annotation = annotation

    ### PUBLIC PROPERTIES ###

    @property
    def annotation(self):
        r'''Gets and sets annotated timespan annotation.

        ..  container:: example

            Gets annotation:

            ::

                >>> annotated_timespan.annotation
                ['a', 'b', 'c', 'foo']

        ..  container:: example

            Sets annotation:

            ::

                >>> annotated_timespan.annotation = 'baz'

        Returns arbitrary object.
        '''
        return self._annotation

    @annotation.setter
    def annotation(self, argument):
        self._annotation = argument



# -*- coding: utf-8 -*-
import abc
from abjad.tools import mathtools
from abjad.tools import systemtools
from abjad.tools.abctools import AbjadValueObject


class IntervalClass(AbjadValueObject):
    '''Interval-class base class.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(self):
        pass

    ### SPECIAL METHODS ###

    def __abs__(self):
        r'''Absolute value of interval-class.

        Returns new interval-class.
        '''
        return type(self)(abs(self._number))

    def __float__(self):
        r'''Changes interval-class to float.

        Returns float.
        '''
        return float(self._number)

    def __int__(self):
        r'''Change interval-class to integer.

        Returns integer.
        '''
        return self._number

    def __str__(self):
        r'''String representation of interval-class.

        Returns string.
        '''
        return self._format_string

    ### PRIVATE PROPERTIES ###

    @property
    def _format_string(self):
        return str(self.number)

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        if type(self).__name__.startswith('Named'):
            values = [str(self)]
        else:
            values = [
                mathtools.integer_equivalent_number_to_integer(float(self))
                ]
        return systemtools.FormatSpecification(
            client=self,
            coerce_for_equality=True,
            repr_is_indented=False,
            storage_format_is_indented=False,
            storage_format_args_values=values,
            template_names=['number'],
            )

    ### PUBLIC PROPERTIES ###

    @property
    def number(self):
        r'''Number of interval-class.

        Returns number.
        '''
        return self._number



# -*- coding: utf-8 -*-


def get_measure_that_starts_with_container(container):
    '''Get measure that starts with `container`.

    Returns measure or none.
    '''
    from abjad.tools import scoretools

    if isinstance(container, scoretools.Container):
        contents = container._get_descendants_starting_with()
        contents = [x for x in contents if isinstance(x, scoretools.Measure)]
        if contents:
            return contents[0]
        raise MissingMeasureError



# -*- coding: utf-8 -*-
import copy
from abjad import *


def test_scoretools_GraceContainer___copy___01():
    r'''Grace containers copy kind.
    '''

    grace_container_1 = scoretools.GraceContainer([Note("d'32")], kind='after')
    grace_container_2 = copy.copy(grace_container_1)

    assert grace_container_1 is not grace_container_2
    assert grace_container_1.kind == grace_container_2.kind == 'after'



# -*- coding: utf-8 -*-
import abc
import copy
import re
from abjad.tools.abctools import AbjadObject
from abjad.tools.topleveltools import new


class GraphvizMixin(AbjadObject):
    r'''An attributed Graphviz mixin.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Graphviz'

    __slots__ = ()

    _word_pattern = re.compile('^\w[\w\-:]*$')

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(self, attributes=None):
        self._verify_attributes(attributes, '_attributes')

    ### PRIVATE METHODS ###

    @staticmethod
    def _copy_with_memo(node):
        from abjad.tools import systemtools
        agent = systemtools.StorageFormatAgent(node)
        edges = set(getattr(node, '_edges', ()))
        mapping = dict()
        arguments = agent.get_template_dict()
        if 'children' not in arguments:
            copied_node = new(node)
            mapping[node] = copied_node
        else:
            copied_node = new(node, children=None)
            mapping[node] = copied_node
            for child in arguments['children']:
                (
                    copied_child,
                    child_edges,
                    child_mapping,
                    ) = GraphvizMixin._copy_with_memo(child)
                copied_node.append(copied_child)
                edges.update(child_edges)
                mapping.update(child_mapping)
        return copied_node, edges, mapping

    def _format_attribute(self, name, value):
        from abjad.tools import graphtools
        result = []
        if isinstance(value, graphtools.GraphvizTable):
            result.extend(str(value).splitlines())
            result[0] = '{}={}'.format(name, result[0])
        else:
            result.append('{}={}'.format(
                name,
                self._format_value(value, quote_keywords=True),
                ))
        result = '\n'.join(result)
        return result

    def _format_attribute_list(self, attributes):
        formatted_attributes = []
        for k, v in sorted(attributes.items()):
            formatted_attributes.append(self._format_attribute(k, v))
        result = []
        for i, formatted_attribute in enumerate(formatted_attributes):
            if i < len(formatted_attributes) - 1:
                formatted_attribute = formatted_attribute + ','
            lines = formatted_attribute.splitlines()
            result.extend(lines)
        for i, line in enumerate(result):
            if 0 < i:
                result[i] = '    ' + line
        result[0] = '[' + result[0]
        result[-1] = result[-1] + '];'
        return result

    def _format_value(self, value, quote_keywords=False):
        if isinstance(value, bool):
            return repr(value).lower()
        elif isinstance(value, (int, float)):
            return repr(value)
        elif isinstance(value, str):
            if value.startswith('<') and value.endswith('>'):
                return value
            should_quote = False
            if not self._word_pattern.match(value):
                should_quote = True
            elif value and value[0].isdigit():
                should_quote = True
            elif value.lower() in (
                'digraph',
                'graph',
                'node',
                'subgraph',
                ) and quote_keywords:
                should_quote = True
            if should_quote:
                value = value.replace('"', r'\"')
                value = '"{}"'.format(value)
            return value
        elif isinstance(value, (list, tuple)):
            return '"{}"'.format(', '.join(
                self._format_value(x, quote_keywords=quote_keywords)
                for x in value))
        raise ValueError

    def _verify_attributes(self, attributes, destination):
        assert isinstance(attributes, (dict, type(None)))
        if attributes is None:
            attributes = {}
        else:
            attributes = copy.copy(attributes)
        setattr(self, destination, attributes)

    ### PUBLIC PROPERTIES ###

    @property
    def attributes(self):
        r'''Attributes of Graphviz object.
        '''
        return self._attributes



# -*- coding: utf-8 -*-
from abjad import *


def test_indicatortools_Tempo___init___01():
    r'''Initializes tempo with integer.
    '''

    tempo = Tempo(Duration(3, 32), 52)
    assert format(tempo, 'lilypond') == '\\tempo 16.=52'


def test_indicatortools_Tempo___init___02():
    r'''Initializes tempo from textual indication.
    '''

    tempo = Tempo(textual_indication='Langsam')
    assert format(tempo, 'lilypond') == '\\tempo Langsam'


def test_indicatortools_Tempo___init___03():
    r'''Initializes tempo with tempo range.
    '''

    tempo = Tempo((1, 8), (52, 57.5))
    assert format(tempo, 'lilypond') == '\\tempo 8=52-57.5'


def test_indicatortools_Tempo___init___04():
    r'''Initializes tempo from text, duration and range.
    '''

    tempo = Tempo(Duration(1, 4), (120, 133), 'Quick')
    assert format(tempo, 'lilypond') == '\\tempo Quick 4=120-133'



#! /usr/bin/env python
import os
from abjad.tools import systemtools


def iterate():
    for directory, subdirectory_names, file_names in os.walk('.'):
        for file_name in file_names:
            if file_name.endswith(('.py', '.ly', '.raw', '.rst')):
                file_path = os.path.join(directory, file_name)
                yield file_path


def ensure_source_file_terminal_blank_lines():
    total_source_file_names_without_blank_lines = 0
    for file_path in iterate():
        with open(file_path, 'r') as file_pointer:
            contents = file_pointer.read()
            if not contents:
                continue
            elif contents[-1] == '\n':
                continue
            total_source_file_names_without_blank_lines += 1
            contents = '{}\n'.format(contents)
            with open(file_path, 'w') as file_pointer:
                file_pointer.write(contents)
    message = 'Total source file_names with terminal blank lines: {}'
    message = message.format(total_source_file_names_without_blank_lines)
    print(message)
    print()


if __name__ == '__main__':
    systemtools.IOManager.clear_terminal()
    print('Ensuring source file-terminal blank lines ...')
    print()
    ensure_source_file_terminal_blank_lines()
